/**
	*	embed signage Ajax function. Allows effecient reuse.
	*	@constructor
	* @param {Object} options - Options allow the user to set the url, request, data, success, error, timeout time, complete , type and sendData for the request. It will be merged into the this.settings variable.
*/

var EmbedAjax = function(options){

	this.settings = {

		"request":false,
		"url":false,
		"data":false,
		"success":false,
		"error":false,
		"timeout":false,
		"complete":false,
		"type":"POST",
		"sendData":false
	};

	this.settings = this.extend(false, this.settings, options);

};

EmbedAjax.prototype.url = function(url){

	try{

		this.settings.url = url;

	}catch(e){

		// console.log("EmbedAjax - Failed to update data [ " + e + " ]  ");

	}

};

EmbedAjax.prototype.success = function(success){

	try{

		if(typeof success === "function"){

			this.settings.success = success;

		}

	}catch(e){

		// console.log("Setting success failed exception [ "+ e +" ]");

	}

};

EmbedAjax.prototype.error = function(error){
	
	try{

		if(typeof error === "function"){

			this.settings.error = error;

		}

	}catch(e){

		// console.log("Setting success failed exception [ "+ e +" ]");

	}

};

EmbedAjax.prototype.complete = function(complete){

	try{

		if(typeof complete === "function"){

			this.settings.complete = complete;

		}

	}catch(e){

		// console.log("Setting success failed exception [ "+ e +" ]");

	}

};
		
EmbedAjax.prototype.data = function(data){

	try{

		this.settings.data = data;

	}catch(e){

		// console.log("EmbedAjax - Failed to update data [ " + e + " ]  ");

	}

};

/*
 *
 * cusotm merge function like jquery;
 *
 */
EmbedAjax.prototype.merge = function (obj, deep, extended) {

	try{

	    for (var prop in obj) {
	        if (Object.prototype.hasOwnProperty.call(obj, prop)) {
	            // If deep merge and property is an object, merge properties
	            if (deep && Object.prototype.toString.call(obj[prop]) === '[object Object]') {
	                extended[prop] = this.extend(true, extended[prop], obj[prop]);
	            } else {
	                extended[prop] = obj[prop];
	            }
	        }
	    }

	}catch(e){

		// console.log("EmbedAjax - Failed to call merge function [ " + e + " ] ");

	}

};

/*
 *
 * Custom extend function like jquery
 *
 */
EmbedAjax.prototype.extend = function () {

	try{

	    // Variables
	    var extended = {};
	    var deep = false;
	    var i = 0;
	    var length = arguments.length;

	    // Check if a deep merge
	    if (Object.prototype.toString.call(arguments[0]) === '[object Boolean]') {
	        deep = arguments[0];
	        i++;
	    }

	    // Loop through each object and conduct a merge
	    for (; i < length; i++) {
	        var obj = arguments[i];
	        this.merge(obj, deep, extended);
	    }

	    return extended;

	}catch(e){

		// console.log("EmbedAjax - Failed to call extend function [ " + e + " ] ");

	}

};

EmbedAjax.prototype.remove = function(){

	try{

		if(this.settings.request){

			this.close();

			for(var objectKey in this.settings){

				delete this.settings[objectKey];

			}

		}

	}catch(e){

		// console.log("EmbedAjax - Failed to call remove function [ " + e + " ] ");

	}

};

EmbedAjax.prototype.close = function(){

	try{

		if(this.settings.request){

			this.settings.success	= function(){};
			this.settings.complete	= function(){};
			this.settings.error	= function(){};

			this.settings.request.close();

		}

	}catch(e){

		// console.log("EmbedAjax - Failed to call close function [ " + e + " ] ");

	}

};

EmbedAjax.prototype.onTimeout = function(){

	// console.log("timeout called");

	try{

		if(this.settings.error){
	        this.settings.error(this.settings.request.responseText);
		}

		if(this.settings.complete){
	        this.settings.complete();
		}

	}catch(e){

		// console.log("EmbedAjax - Failed to call timeout function [ " + e + " ] ");

	}

};

EmbedAjax.prototype.onLoad = function(){

	// try{

	if (this.settings.request.status >= 200 && this.settings.request.status < 400) {

			if(this.settings.success){
		        this.settings.success(this.settings.request.responseText);
			}

			if(this.settings.complete){
		        this.settings.complete();
			}

    } else {

        	if(this.settings.error){
    	        this.settings.error(this.settings.request.responseText);
    		}

    		if(this.settings.complete){
    	        this.settings.complete();
    		}

    }

	// }catch(e){

	// 	console.log("EmbedAjax - Failed to call onLoad function [ " + e + " ] ");

	// }

};

EmbedAjax.prototype.onError = function(){

	// console.log("onError called");

	try{

		if(this.settings.error){
	        this.settings.error(this.settings.request.responseText);
		}

		if(this.settings.complete){
	        this.settings.complete();
		}

	}catch(e){

		// console.log("EmbedAjax - Failed to call onError function [ " + e + " ] ");

	}

};

EmbedAjax.prototype.connect = function () {

	// console.log("Calling connect embed ajax");

	try{

	    if (typeof this.settings.data === "object") {
	        this.settings.sendData = this.mapPostKeys(this.settings.data);
	    }

			// console.log("Checking for XMLHttpRequest");
	    if(!this.settings.request){
	        this.settings.request = new XMLHttpRequest();
	    }

			// console.log("Calling open");
	    this.settings.request.open(this.settings.type, this.settings.url, true);

			// console.log("Setting timeout");
	    if(this.settings.timeout){
				// console.log("timeout exists so will set timeout");
				// console.log("Timeout length is " + this.settings.timeout);
	    	this.settings.request.timeout = this.settings.timeout;
	    }

	    if (this.settings.type === "POST") {
	    	this.settings.request.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded; charset=UTF-8');
	    	this.settings.request.send(this.settings.sendData);
	    } else {
	    	this.settings.request.send();
	    }

		// console.log("Setting on load");
	    this.settings.request.onload = this.onLoad.bind(this);

		// console.log("Setting on error");
	    this.settings.request.onerror = this.onError.bind(this);

		// console.log("Setting on timeout");
		this.settings.request.ontimeout = this.onTimeout.bind(this);

	}catch(e){

		// console.log("EmbedAjax - Failed to call connect function [ " + e + " ] ");

	}

};

/*

Ported jquery param to pure javascript

*/
EmbedAjax.prototype.buildParams = function (prefix, obj, traditional, add) {

	try{

		   if (Array.isArray(obj)) {

		       // Serialize array item.
		       for (var objKey in obj) {

		           if (traditional || /\[\]$/.test(prefix)) {

		               add(prefix, obj[objKey]);

		           } else {

		               this.buildParams(prefix + "[" + ( typeof obj[objKey] === "object" || Array.isArray(obj[objKey]) ? objKey : "" ) + "]", obj[objKey], traditional, add);

		           }

		       }

		   } else if (!traditional && obj !== null && typeof obj === "object") {

		       // Serialize object item.
		       for (var objKey2 in obj) {

		           this.buildParams(prefix + "[" + objKey2 + "]", obj[objKey2], traditional, add);

		       }

		   } else {
		       // Serialize scalar item.
		       add(prefix, obj);

		   }

	}catch(e){

		// console.log("EmbedAjax - Failed to call buildParams function [ " + e + " ] ");

	}

};


/*
 *
 * Map post keys function like jqueyr
 *
 */
EmbedAjax.prototype.mapPostKeys = function (a) {

	try{

	    var s = [], add = function (key, value) {

	        s[s.length] = encodeURIComponent(key) + "=" + encodeURIComponent(value);

	    };

	    for (var prefix in a) {

	        this.buildParams(prefix, a[prefix], false, add);

	    }

	    // Return the resulting serialization
	    return s.join("&").replace(/%20/g, "+");

	}catch(e){

		// console.log("EmbedAjax - Failed to mapPostKeys function [ " + e + " ] ");

	}

};
