var EmbedApp = function()
{

    // document.addEventListener( 'deviceready', this.init.bind( this ), false );

};

EmbedApp.prototype.config = {

    'url'       : 'https://devices.embedsignage.com',
    'storageUrl': 'https://storage.embedsignage.com',
    'wsUrl'     : 'wss://websockets.embedsignage.com:443',
    'version'   : '2.0.7'

};

EmbedApp.prototype.init = function() 
{

    this.domEl = {
        'app' : document.getElementById('app'),
        'content' : document.getElementById('content'),
        'noContent' : document.getElementById('noContent'),
        'version'   : document.getElementById('versionNumber'),
        'serial'   : document.getElementById('serialNumber'),
        'name'      : document.getElementById('deviceName'),
        'account'   : document.getElementById('accountName')
    };

    this.domEl.version.innerText = 'v' + this.config.version;

    this.loadModules();

    setTimeout( this.run.bind(this), 10000 );

    // this.run();

};

EmbedApp.prototype.features = ['appServer'];

EmbedApp.prototype.hasFeature = function(feature)
{
    if( this.features.indexOf(feature) != -1 )
    {
        return true;
    }

    return false;
};

EmbedApp.prototype.setupAppServer = function(path, port)
{
    return new PlaybackAppStaticServer(path, port);
};

EmbedApp.prototype.loadModules = function()
{

    // window.console   = new EmbedConsole();

    this.brightSign = new BrightSign();
    this.settings   = new Settings();

    this.notifications = new EmbedNotification();

    this.file       = new EmbedFileManager();
    this.license    = new EmbedLicensing();

    this.messages   = new EmbedMessages();

    this.device     = new EmbedDevice();

    this.connect    = new EmbedConnect();

    this.channel    = new EmbedChannel();
    this.layout     = new EmbedLayout();

    this.fs = require("fs"); 

};

EmbedApp.prototype.run = function()
{

    app.brightSign.classes.registry.write('html', 'use-brightsign-media-player', '1').then( this.chromeVideoPlayerResponse.bind(this) ).catch( this.chromeVideoPlayerError.bind(this) );

    if( app.brightSign.registryData && app.brightSign.registryData.embedinfo && app.brightSign.registryData.embedinfo.setupcomplete )
    {
        this.localDataLoaded();    
    }
    else
    {
        this.startSetup();
    }
    
};

EmbedApp.prototype.chromeVideoPlayerResponse = function(data)
{
    // console.log('chrome video player response: ' + JSON.stringify(data));
};

EmbedApp.prototype.chromeVideoPlayerError = function(data)
{
    // console.log('chrome video player error: ' + JSON.stringify(data));
};

EmbedApp.prototype.startSetup = function()
{

    this.updateInfo();

    if( app.brightSign.availableInterfaces && app.brightSign.availableInterfaces[0] )
    {
        app.messages.messages.setup.description = 'You need to set up your player, visit this address in your browser. http://' + app.brightSign.availableInterfaces[0].address + ':8000';
        app.messages.show('setup');
    }
    else
    {
        app.messages.messages.setup.description = 'You need to set up your player, visit this address in your browser. No address found.';
        app.messages.show('setup');

        setTimeout( this.startSetup.bind(this), 3000 );
    }
    

};

EmbedApp.prototype.fileSystemReady = function()
{

};

EmbedApp.prototype.localDataLoaded = function()
{

    var ready = true;

    this.device.details.stats.bootTime = new Date().toString();

    if( this.geolocationTimeout )
    {
        clearTimeout(this.geolocationTimeout);
    }

    if( !this.device.geolocationFinished )
    {   
        ready = false;

        this.geolocationTimeout = setTimeout( this.localDataLoaded.bind(this), 2000 );
    }

    if( ready )
    {

        this.updateInfo();

        this.controller = new EmbedController(this.config.wsUrl, this.device.details.embedID, this.device.accountKey);
        
        this.controller.onGetScreenshot = this.getScreenshot.bind(this);
        this.controller.onGetDeviceInfo = this.getControllerInfo.bind(this);

        this.controller.onSetTime       = this.brightSign.setDate.bind(this.brightSign);

        this.controller.onReboot = this.brightSign.reboot.bind(this.brightSign)
        this.controller.onGetLogs = this.getLogs.bind(this);

        this.channel.checkSchedule();
        this.connect.load();

        if( this.device.CMSData && this.device.CMSData.device && this.device.CMSData.device.settings )
        {
            this.setRebootSchedule('', this.device.CMSData.device.settings.rebootSchedule);
        }

        if( this.device.CMSData && this.device.CMSData.device && this.device.CMSData.device.settings && this.device.CMSData.device.settings.cecSchedule )
        {
            this.setCecSchedule('', this.device.CMSData.device.settings.cecSchedule);
        }

        if( this.device.CMSData && this.device.CMSData.usesWebsockets )
        {
            this.controller.enable();
        }

    }

};

EmbedApp.prototype.updateInfo = function()
{

    if( this.device.CMSData && this.device.CMSData.device )
    {

        if( this.device.CMSData.device.name && typeof this.device.CMSData.device.name != 'undefined' )
        {
            this.domEl.name.parentNode.style.display = 'block';
            this.domEl.name.innerText = this.device.CMSData.device.name;
        }
        else
        {
            this.domEl.name.parentNode.style.display = 'none';
        }

        if( this.device.CMSData.device.account && typeof this.device.CMSData.device.account.company_name && typeof this.device.CMSData.device.account.company_name != 'undefined' )
        {
            this.domEl.account.parentNode.style.display = 'block';
            this.domEl.account.innerText = this.device.CMSData.device.account.company_name; 
        }
        else
        {
            // this.domEl.account.innerText = ;
            this.domEl.account.parentNode.style.display = 'none';
        }

    }

    if( this.device.details && this.device.details.serial )
    {
        this.domEl.serial.parentNode.style.display = 'block';
        this.domEl.serial.innerText = this.device.details.serial; 
    }
    else
    {
        // this.domEl.account.innerText = ;
        this.domEl.serial.parentNode.style.display = 'none';
    }

    this.device.getScreenInfo();

};

EmbedApp.prototype.updateVideoPlayer = function(newValue)
{

    if( app.brightSign.registryData.embedinfo.videoplayer != newValue )
    {
        app.brightSign.classes.registry.write('embedinfo', 'videoplayer', newValue).then( this.videoPlayerChanged.bind(this) ).catch( app.brightSign.promiseError.bind(app.brightSign) );
    }   

};

EmbedApp.prototype.videoPlayerChanged = function()
{
    app.brightSign.reboot();
};

EmbedApp.prototype.getScreenshot = function( callback )
{

    if( !this.screenshotClass )
    {
        var ScreenshotClass = require("@brightsign/screenshot");
        this.screenshotClass = new ScreenshotClass();
    }
    
    this.screenshotClass.syncCapture({
        'fileName'  : 'SD:/screen.jpg',
        'quality'   : 75
    });

    var body = this.fs.readFileSync('/storage/sd/screen.jpg');

    var base64 = body.toString('base64');

    var request = new Request({
        'url'   : this.config.url + '/api/v2/' + app.device.accountKey  + '/uploadDevicePreview',
        'type'  : 'POST',
        'data'	: {
        	'base64data' : base64, 'deviceid' : this.device.details.embedID, 'type' : 'jpeg'
        }
    });

    request.onSuccess = callback;

};

EmbedApp.prototype.screenshotSuccess = function()
{
    this.callback();
};

EmbedApp.prototype.createMessagePort = function()
{
	
	this.bsMessagePort = new BSMessagePort();
	
	this.bsMessagePort.onbsmessage = this.receiveBrightscriptMessage.bind(this);
};

EmbedApp.prototype.receiveBrightscriptMessage = function(msg)
{
	
	if( msg.data.message == 'logsStored' )
	{

        this.logsStored();

	}

};

EmbedApp.prototype.logsStored = function()
{
    if( typeof this.logsCallback == 'function' )
    {

        var body = this.fs.readFileSync('/storage/sd/log.txt');

        var base64 = body.toString('base64');

        var request = new Request({
            'url'   : this.config.url + '/api/v2/' + app.device.accountKey  + '/uploadlogs',
            'type'  : 'POST',
            'data'	: {
                'base64data' : base64, 'deviceid' : this.device.details.embedID, 'type' : 'txt'
            }
        });

        request.onSuccess = this.logsSent.bind(this);
    }
};

EmbedApp.prototype.logsSent = function()
{
    if( typeof this.logsCallback == 'function' )
    {
        this.logsCallback();

        this.logsCallback = null;
    }

};

EmbedApp.prototype.getLogs = function( callback )
{

    if(!this.bsMessagePort)
    {
        this.createMessagePort();
    }

    this.bsMessagePort.PostBSMessage({
        module: 'Logs',
        action: 'get',
    });

    this.logsCallback = callback;
};

EmbedApp.prototype.getLogsSuccess = function()
{
    this.callback();
};

EmbedApp.prototype.getControllerInfo = function( callback )
{

    callback( {
        'screenOn'  : true,
        'volume'    : 'N/A',
        'time'      : this.getDeviceTime()
    } );

};

EmbedApp.prototype.getDeviceTime = function()
{

    try{

        var time = new Date();

        // var timeObj = {
        //         "year":0,
        //         "month":1,
        //         "day":2,
        //         "hour":3,
        //         "minute":4,
        //         "second":5
        // };

        // var timeArr = time.split(":");

        var year = parseInt(time.getFullYear());
        var month = parseInt(time.getMonth());
        var day = parseInt(time.getDate());
        var hour = parseInt(time.getHours());
        var minute = parseInt(time.getMinutes());
        var second = parseInt(time.getSeconds());

        var monthArr = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sept", "Oct", "Nov", "Dec"];

        if(hour < 10){
            hour = "0" + hour;
        }

        if(minute < 10){
            minute = "0" + minute;
        }

        if(second < 10){
            second = "0" + second;
        }

        var dateString = monthArr[month] + " " + this.formatDay(day) + " " + year + " " + hour + ":" + minute + ":" + second;

        return dateString;

    }catch(e){

    }

};

EmbedApp.prototype.formatDay = function( day ) 
{
    
    var dayString = day.toString();

    var lastDigit   = dayString.substr( dayString.length - 1, 1 );
    var firstDigit  = dayString.substr( 0, 1 );

    if( dayString.length == 2 && firstDigit == 1 )
    {
        day = day + 'th';
    }
    else
    {
        switch( parseInt(lastDigit) )
        {
            case 1 :
                day = day + 'st';
            break;

            case 2 :
                day = day + 'nd';
            break;

            case 3 :
                day = day + 'rd';
            break;

            default :
                day = day + 'th';
            break;

        }
    }

    return day;

};

EmbedApp.prototype.removeAction = function( action, callback )
{
    if( this.device.details.embedID )
    {

        var request = new Request({
            'url'   : this.config.url + '/api/v2/' + app.device.accountKey + '/devices/removeAction',
            'type'  : 'POST',
            'data'  : { 
                'deviceid' : app.device.details.embedID,
                'deviceID' : app.device.details.embedID,
                'action'    : action
            }
        });

        request.onSuccess = callback;

        // request.onError = this.requestError.bind(this);

    }
    
};

EmbedApp.prototype.clearCache = function( callback )
{   
    try {
        app.file.emptyDirectory('info/player', function( success ) {

            app.file.emptyDirectory('info/app', function( success ) {
                callback();
            });
    
        });
    }
    catch(e)
    {
        callback();
    }
    
};

EmbedApp.prototype.troubleshoot = function( type )
{

    this.troubleshooting = true;

    if( type && type == 'local' )
    {

        try {


            this.layout.show( false );

            app.messages.show( 'troubleshooting' );

            this.clearCache(function() {
                app.channel.clear( function() {
                    setTimeout( app.brightSign.reboot.bind(app.brightSign), 10000 );
                } );
            });


        } 
        catch(e)
        {
            setTimeout( app.brightSign.reboot.bind(app.brightSign), 2000 );

        }

    }
    else
    {

        try {

            this.removeAction( "runTroubleshooter", function() {

                this.layout.show( false );

                app.messages.show( 'troubleshooting' );

                this.clearCache(function() {
                    app.channel.clear( function() {

                        setTimeout( app.brightSign.reboot.bind(app.brightSign), 10000 );

                    } );
                });

            }.bind(this) );

        } 
        catch(e)
        {
            setTimeout( app.brightSign.reboot.bind(app.brightSign), 2000 );

        }

    }


};

EmbedApp.prototype.reset = function()
{

    this.troubleshooting = true;

    this.layout.show( false );

    app.messages.show( 'troubleshooting' );

    this.clearCache(function() {
        app.channel.clear( function() {
        
            app.device.clearCMSData( function() {

                app.brightSign.classes.registry.write({
                    'embedinfo' : null,
                    'embedauth' : null,
                    'embednetwork' : null
                }).then( this.resetComplete.bind(this) ).catch( this.resetComplete.bind(this) );

            }.bind(this) );

        }.bind(this) );
    }.bind(this));

    // registries.. embedinfo, embedauth, embednetwork



};

EmbedApp.prototype.resetComplete = function()
{

    setTimeout( app.brightSign.reboot.bind(app.brightSign), 2000 );


};

EmbedApp.prototype.processActions = function () 
{

    var actions = this.device.CMSData.device.actions;

    this.device.CMSData.device.actions = [];

    if( actions instanceof Array ) 
    {

        for( var actionKey in actions ) 
        {

            switch( actions[actionKey] ) 
            {

                case "updateFirmware" :

                    this.updateFirmware();                    
                    // this.samsung.apis.samsungUpdater.checkForUpdate();

                break;

                case "updateApplication" :

                    this.updateApplication();
                    // this.samsung.apis.samsungUpdater.checkForUpdate();

                break;

                case "runTroubleshooter" :

                    this.troubleshoot();

                break;

                case "refreshContent" :

                    this.channel.checkSchedule( true );

                break;

            }

        }

    }

};

EmbedApp.prototype.setRebootSchedule = function( oldSchedule, newSchedule )
{

    if( oldSchedule == '' || JSON.stringify(oldSchedule) != JSON.stringify(newSchedule) )
    {


        // console.log('setting rebootSchedule');

        if( this.rebootTimeout )
        {

            clearTimeout(this.rebootTimeout);

            this.rebootTimeout = false;

        }

        if( newSchedule && newSchedule.interval != 'none' )
        {

            this.rebootTimeout = setTimeout(this.checkRebootSchedule.bind(this), 60000);

        }

    }

};

EmbedApp.prototype.checkRebootSchedule = function()
{

    // console.log('checking rebootSchedule');

    var days = ['sunday', 'monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday'];
    var schDays = ['sunday', 'monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday'];

    if( this.device.CMSData.device.settings.rebootSchedule.interval == 'weekly' )
    {

        schDays = [ this.device.CMSData.device.settings.rebootSchedule.day ];

    }

    var date = new Date();

    var day = date.getDay();
    var hour = date.getHours();
    var min = date.getMinutes();

    var schTime = this.device.CMSData.device.settings.rebootSchedule.time.split(':');

    if( schDays.indexOf( days[ day ] ) != -1 && hour == schTime[0] && min == schTime[1] )
    {

        var bootTime = new Date(this.device.details.stats.bootTime);

        // console.log(bootTime, date);

        if( ( date.getTime() - bootTime.getTime() ) > 60000 )
        {

            this.brightSign.reboot();

        }
        else
        {
            // console.log('reboot but only just done');
        }

    }

    this.rebootTimeout = setTimeout(this.checkRebootSchedule.bind(this), 60000);

};

EmbedApp.prototype.removeRebootSchedule = function()
{

    if( this.rebootTimeout )
    {

        clearTimeout(this.rebootTimeout);

        this.rebootTimeout = false;

    }

};


EmbedApp.prototype.setCecSchedule = function( oldSchedule, newSchedule )
{

    if( oldSchedule == '' || JSON.stringify(oldSchedule) != JSON.stringify(newSchedule) )
    {


        // console.log('setting rebootSchedule');

        if( this.cecTimeout )
        {

            clearTimeout(this.cecTimeout);

            this.cecTimeout = false;

        }

        if( newSchedule && newSchedule.length )
        {

            this.cecTimeout = setTimeout(this.checkCecSchedule.bind(this), 60000);

        }

    }

};

EmbedApp.prototype.checkCecSchedule = function()
{

    // console.log('checking rebootSchedule');

    var date = new Date();

    var day = date.getDay();
    var hour = date.getHours();
    var min = date.getMinutes();

    console.log('check cec schedule');
    for( var i = 0; i < this.device.CMSData.device.settings.cecSchedule.length; i++ )
    {

        var schTime = this.device.CMSData.device.settings.cecSchedule[i].time.split(':');
        
        console.log('schTime: ' + schTime[0] + ':' + schTime[1] + ' time: ' + hour + ':' + min);
        if( hour == schTime[0] && min == schTime[1] )
        {
        
            this.brightSign.cecSend({cmd: this.device.CMSData.device.settings.cecSchedule[i].type});

        }

    }

    this.cecTimeout = setTimeout(this.checkCecSchedule.bind(this), 60000);

};

EmbedApp.prototype.removeCecSchedule = function()
{

    if( this.cecTimeout )
    {

        clearTimeout(this.cecTimeout);

        this.cecTimeout = false;

    }

};


EmbedApp.prototype.updateApplication = function() 
{

    var downloadAttempts = 0;

    this.removeAction( "updateApplication", function() {

        app.file.downloadFile( this.config.url + '/channels/newdevice/brightsign/latest/apps/autorun.zip', 'updates', 'autorun.zip', function( success, error ) {

            if( success )
            {
                // console.log('downloaded autorun.zip');

                app.file.downloadFile(this.config.url + '/channels/newdevice/brightsign/latest/setup/autorun.brs', 'updates', 'autorun.brs', function( success, error ) {

                    if( success )
                    {
                        this.moveUpdateFiles();

                        
                        // downloaded
                    }
                    else
                    {

                        // if( downloadAttempts == 3 )
                        // {

                            app.notifications.send({
                                'type' : 'error',
                                'title' : 'Could not download application',
                                'message' : 'Failed to download application'
                            });

                            app.messages.show('downloadFailed');

                        // }
                        // else
                        // {
                        //     downloadAttempts++;
                        // }

                    }

                }.bind(this), function(progress) {
                    // console.log(progress);
                } );
                // downloaded
            }
            else
            {

                // if( downloadAttempts == 3 )
                // {

                    app.notifications.send({
                        'type' : 'error',
                        'title' : 'Could not download application',
                        'message' : 'Failed to download application'
                    });

                    app.messages.show('downloadFailed');

                // }
                // else
                // {
                //     downloadAttempts++;
                // }

            }

        }.bind(this), function(progress) {
            // console.log(progress);
        } );

    }.bind(this));
};

EmbedApp.prototype.moveUpdateFiles = function() 
{
    app.file.moveFile( 'updates', 'autorun.zip', 'root', 'autorun.zip', function( success ) {

        app.file.moveFile( 'updates', 'autorun.brs', 'root', 'autorun.brs', function( success ) {

            app.channel.clear( function() {

                // console.log('setting reboot after moveUpdateFiles');
                setTimeout( app.brightSign.reboot.bind(app.brightSign), 2000 );

            } );
            
        }.bind(this) );
        
    }.bind(this) );

    
};

EmbedApp.prototype.loadAnalytics = function(embedID, licenseKey){

    try{

        if (window.EA) {

            // console.log("EA already exists");

            window.EA.setAPIKey(licenseKey);

            return;
            
        }
        
        // console.log("Load Analytics [ "+ embedID +", "+ licenseKey +" ] ");

        if (!licenseKey) {
            // console.log("No license key");
            return;
        }

        if (embedID === null && embedID === '') {
            // console.log("No embedID");
            return;
        }

        window.EA = new EmbedAnalytics({
            'apiKey': licenseKey,
            'onlineOffline': true,
            'audienceMeasurement':false,
            'debug': false,
            'deviceId': embedID
        });
        
        window.EA.onread = function(file, callback) {

            // console.log("Analytics on read");

            app.brightSign.classes.registry.read(file, file)
            .then(function(data){

                // console.log("Read promise true!");
                // console.log(data);

                if(data){
                    callback(data);
                }else{
                    callback(false);
                }

            }, 
            function(){
                
                callback(false);

            }).catch(function(e){

                callback(false);

            });


        }.bind(this);

        window.EA.onwrite = function(file, data, callback) {

            // console.log(data);

            app.brightSign.classes.registry.write(file, file, data)
            .then(function(){

                // console.log("Write succes analytics");

                callback(true);

            }, function(){

                callback(false);

            }).catch(function(e){

                // console.log("Wrtie failed");
                // console.log(e);
                callback(false);

            });

        }.bind(this);

        window.EA.start();

    }catch(e){

        // console.log("Load analytics Failed");
        // console.log(e);

    }

};

EmbedApp.prototype.updateFirmware = function() 
{

    var request = new Request({
        'url'   : this.config.url + '/api/v2/' + app.device.accountKey + '/devices/firmware',
        'type'  : 'POST',
        'data'  : { 
            'deviceid' : app.device.details.embedID,
            'deviceID' : app.device.details.embedID,
        }
    });

    request.onSuccess = this.updateFirmwareSuccess.bind(this);

        // request.onError = this.requestError.bind(this);

    
};

EmbedApp.prototype.updateFirmwareSuccess = function( response )
{
    var downloadAttempts = 0;
    var firmwareLocation = '';


    this.removeAction( "updateFirmware", function() {

        if( response.success && response.latest )
        {

            app.file.downloadFile(response.latest, 'updates', 'firmware.bsfw', function( success, error ) {

                if( success )
                {
                    this.moveFirmwareUpdateFiles();
                }
                else
                {

                    // if( downloadAttempts == 3 )
                    // {

                        app.notifications.send({
                            'type' : 'error',
                            'title' : 'Could not download firmware',
                            'message' : 'Failed to download firmware'
                        });

                        app.messages.show('downloadFailed');

                    // }
                    // else
                    // {
                    //     downloadAttempts++;
                    // }

                }

            }.bind(this), function(progress) {
                // console.log(progress);
            } );

        }
        else
        {
            app.notifications.send({
                'type' : 'error',
                'title' : 'Could not find firmware for model ' + app.device.details.model,
                'message' : 'Could not find firmware for model ' + app.device.details.model
            });
        }

    }.bind(this));
}

EmbedApp.prototype.moveFirmwareUpdateFiles = function() 
{
    app.file.moveFile( 'updates', 'firmware.bsfw', 'root', 'firmware.bsfw', function( success ) {

        setTimeout( app.brightSign.reboot.bind(app.brightSign), 2000 );
        
    }.bind(this) );

    
};


var PlaybackAppStaticServer = function (path, port) {

    this.libs = {
    };

    this.live = {
        server: null,
		app: null,
        supported: false
    }

    this.loadLibs();

    this.serve(path, port);

};

PlaybackAppStaticServer.prototype.loadLibs = function () {

    try{
        
		// // Load available modules
		// if( ['BrightSign'].indexOf(player.device.info.type) != -1 )
		// {
			var process = require("process");
    		process.chdir("/storage/sd");
		// }

        this.libs.fs = require('fs');
        // this.libs.http = require('http');// Dont need
        // this.libs.url = require('url');// Dont need
        this.libs.path = require('path');
		this.libs.express = require('express');
        this.live.supported = true;

    }catch(e){
        this.live.supported = false;
    }

};

PlaybackAppStaticServer.prototype.stop = function () {

    if (this.live.server) {
        this.live.server.close();
        this.live.server = null;
    }

};

PlaybackAppStaticServer.prototype.serve = function (path, port) {

    if (!this.live.server && this.live.supported) {

		this.live.app = this.libs.express();

		// if( ['BrightSign'].indexOf(player.device.info.type) != -1 )
		// {
			this.live.app.use(this.libs.express.static('./files/apps/'));
        // }

		this.live.app.get('/', function(req, res) {
			res.send('Hello World!');
		});

		this.live.server = this.live.app.listen(parseInt(port), function() {
			console.log("Example app listening on port" + port);
		});

        console.log("Server listening on port " + port);

    }
};