var BrightSign = function()
{

    this.init();

};

BrightSign.prototype.families = {
	"s5" : [
		"raptor", // XC5
		"cobra", // HD5, LS5, XD5
		"sebring", // AU5
	],
	"s4" : [
		"pagani", // LS4, HD4, HS4
		"malibu" // XD4, XT4
	],
	"s3" : [
		"pantera", // LS3, HD3, HS3, HO
		"ranchero", // XD3
		"impala" // XT3
	]
};

// BrightSign.prototype.fullResModels = ["XC4055","XC2055","XT1144", "XT244", "XT245", "XT1145", "XT2145"];
BrightSign.prototype.fullResModels = ["XC4055","XC2055", "XT245", "XT1145", "XT2145"];
BrightSign.prototype.multiOutputModels = ["XC4055","XC2055", "XT2145"];

BrightSign.prototype.availableOutputs = [];
BrightSign.prototype.output = 'hdmi';

BrightSign.prototype.checkFullRes = function()
{

    if( ['3840x2160x50p', '3840x2160x59.94p', '3840x2160x60p', '4096x2160x50p', '4096x2160x59.94p', '4096x2160x60p'].indexOf(this.preferredScreenMode.modeName) == -1 )
    {
        if( this.fullResModels.indexOf('XT1143') == -1 )
        {
            this.fullResModels.push( "XT1143");
        }

        if( this.fullResModels.indexOf('XT243') == -1 )
        {
            this.fullResModels.push( "XT243");
        }
    }
    else
    {   
        var xt3index = this.fullResModels.indexOf("XT1143");

        if( xt3index != -1 )
        {
            this.fullResModels.splice(xt3index, 1);
        }

        var xt23index = this.fullResModels.indexOf("XT243");

        if( xt23index != -1 )
        {
            this.fullResModels.splice(xt23index, 1);
        }
    }

    if( this.fullResModels.indexOf(app.device.details.model) != -1 )
    {
        return true;
    }

    return false;
}

BrightSign.prototype.init = function()
{

    // check local storage/filesystem for resolution.
    // Set videooutput to maximum
    // 
    var process = require("process");
    process.chdir("/storage/sd");

    this.loadModules();

    // this.loadSettingsPage();
    // 

    this.classes.registry.read().then( function( info ) {

        this.registryData = info;

        // console.log(this.registryData);

        this.checkStats();

    }.bind(this) ).catch( function(e) {

        this.registryData = false;

        this.checkStats();

    }.bind(this) );

    

};

BrightSign.prototype.checkStats = function() 
{

    // console.log(this.registryData);

    this.localTime = new Date();
    this.localTimeOffset = ( this.localTime.getTimezoneOffset() / 60 ) * -1;

    // console.log(this.localTimeOffset);


    this.detectScreen();
    // this.detectAudio();
    this.detectNetwork();
    this.detectCPU();
    this.detectRAM();

    this.detectStorage();

    // this.detectDecoderConfig();

    clearTimeout(this.statsTimeout);
    this.statsTimeout = setTimeout( this.checkStats.bind(this), 60000 );

};

BrightSign.prototype.detectDecoderConfig = function()
{
    // app.brightSign.classes.decoderConfig.getConfig().then(function(response) {
    //     this.decoderConfig = response;
    // }.bind(this)).catch(function() {

    // });
}

BrightSign.prototype.loadModules = function()
{

    this.modules = {
        'videoModeConfig'   : require("@brightsign/videomodeconfiguration"),
        // 'decoderConfig'     : require("@brightsign/decoderconfiguration"),
        'videoOutput'       : require("@brightsign/videooutput"),
        'system'            : require("@brightsign/system"),
        'registry'			: require("@brightsign/registry"),
        'networkConfig'		: require("@brightsign/networkconfiguration"),
        'systemTime'        : require("@brightsign/systemtime"),
        // 'assetPool' 		: require("@brightsign/assetpool"),
    };

    this.classes = {
        'videoModeConfig'   : new this.modules.videoModeConfig(),
        // 'decoderConfig'     : new this.modules.decoderConfig(),
        'system'            : new this.modules.system(),
        'registry'			: new this.modules.registry(),
        'network' : {
        	'ethernet'			: new this.modules.networkConfig("Ethernet"),
        	'wifi'				: new this.modules.networkConfig("Wifi"),
        	'modem'				: new this.modules.networkConfig("Modem")
        },
        'os' : require('os'),
        'dns' : require('dns'),
        // 'assetPool' : new this.modules.assetPool('SD:/'),
        'disk'      : require('diskspace'),
        'time'      : new this.modules.systemTime()
    };

    // try {
    //     var videoOutput = new this.modules.videoOutput("hdmi");
    //     videoOutput.setBackgroundColor('000000');
    // }
    // catch(e)
    // {
    //     console.log(e);
    // }

};

BrightSign.prototype.applyConfig = function( network, auth, resolution, multiOutputs )
{

    this.updateCredentials( auth, true );

    this.updateNetwork( network, true );

    if( resolution )
    {
        this.overwriteResolution(resolution, true);
    }

    if( multiOutputs )
    {
        this.overwriteOutputs( multiOutputs );
    }

    // this.updateAudioOutput( audio, true );

    this.classes.registry.write('embedinfo', 'setupcomplete', 'true').then( this.registryWritten.bind(this) ).catch( this.promiseErrorConfig.bind(this) );

    setTimeout(this.reboot.bind(this), 5000);

};

BrightSign.prototype.cecSuccess = function(data) 
{
    console.log(data);
};

BrightSign.prototype.retryCec = function(data) 
{
    console.log(data);
};

BrightSign.prototype.cecSend = function(action, retry)
{
    if( !this.modules.cec )
    {
        this.modules.cec = require("@brightsign/cec");
    }

    if( !this.classes.cec )
    {
        this.classes.cec = new this.modules.cec();
    }

    if(!retry)
    {
        retry = 0;
    }
    else if(retry > 2)
    {
        return;
    }

    var command = [];

    console.log('Sending cec retries: ' + retry);
    switch(action.cmd)
    {
        case 'hdmi_1' :
            console.log('Sending cec hdm1');

            if(retry == 0)
            {
                // Samsung
                command = [0x4F,0x82,0x10,0x00];
            }
            else
            {
                // Samsung
                command = [0x4F,0x82,0x10,0x00];
            }

            
        break;

        case 'hdmi_2' :
            console.log('Sending cec hdm2');
            if(retry == 0)
            {
                // Samsung
                command = [0x4F,0x82,0x20,0x00];
            }
            else
            {
                // Samsung
                command = [0x4F,0x82,0x20,0x00];
            }
            
        break;

        case 'hdmi_3' :
            console.log('Sending cec hdm3');
            if(retry == 0)
            {
                // Samsung
                command = [0x4F,0x82,0x30,0x00];
            }
            else
            {
                // Samsung
                command = [0x4F,0x82,0x30,0x00];
            }
            
        break;

        case 'screen_off' :
            console.log('Sending cec screen_off');
            
            if(retry == 0)
            {
                // Samsung
                command = [0x40,0x36];
            }
            else
            {
                // Samsung
                command = [0x40,0x36];
            }
        break;

        case 'screen_on' :
            console.log('Sending cec screen_on');
            
            if(retry == 0)
            {
                // Samsung
                command = [0x40,0x0D];
            }
            else
            {
                // Samsung
                command = [0x40,0x0D];
            }
        break;

        case 'custom' :
            console.log('Sending cec custom: ' + action.value);
            command = [action.value];
        break;
    }

    this.classes.cec.send(command).then(this.cecSuccess.bind(this)).catch(this.cecSend.bind(this, action, retry + 1));
};

BrightSign.prototype.updateCredentials = function( details, setup )
{

    var bcrypt = require('bcrypt-nodejs');

    var originalUsername = 'admin';
    var originalPassword = bcrypt.hashSync( app.device.details.serial, null, null );

    if( this.registryData.embedauth && this.registryData.embedauth.password )
    {

        // originalUsername = this.registryData.embedauth.username;
        originalPassword = this.registryData.embedauth.password;

    }

    var changes = false;

    if( bcrypt.compareSync( details.old.password, originalPassword ) )
    {

        // console.log('old details match');

        var newHash = bcrypt.hashSync( details.new.password );

        if( originalPassword != newHash )
        {

            this.classes.registry.write('embedauth', {
                'password' : newHash
            }).then( this.registryWritten.bind(this) ).catch( this.promiseErrorAuth.bind(this) );

            changes = true;

        }

    }

    if( changes && !setup )
    {
        // console.log('setting reboot after updateCredentials');
        setTimeout( this.reboot.bind(this), 2000 );
    }

    return changes;

};

BrightSign.prototype.updateDWSConfig = function()
{

    var DWSConfigurationClass = require("@brightsign/dwsconfiguration");
    var dwsConfig = new DWSConfigurationClass();

    var originalUsername = 'admin';
    var originalPassword = bcrypt.hashSync( app.device.details.serial, null, null );

    if( this.registryData.embedauth && this.registryData.embedauth.username && this.registryData.embedauth.password )
    {

        originalUsername = this.registryData.embedauth.username;
        originalPassword = this.registryData.embedauth.password;

    }

    var config = {
        'port' : 80,
        'password' : {
            'value'         : originalPassword,
            'obfuscated'    : true
        },
        'authenticationList' : ["basic"]
    };

    dwsConfig.applyConfig( config ).then( function(e) {
        // console.log(e);


    } ).catch(function(e) { console.log(e); });

};

BrightSign.prototype.overwriteOutputs = function( outputs, setup = false, cms = false )
{

    if( !outputs )
    {
        return;
    }

    this.outputsOverride = outputs;

    if( this.registryData && !this.registryData.embedinfo )
    {
        this.registryData.embedinfo = {};
    }

    this.registryData.embedinfo.outputsoverride = this.outputsOverride;

    app.device.details.stats.multiOutputsCMS = ( cms ? 'cms' : 'local' );

    this.classes.registry.write('embedinfo', 'outputsoverridecms', ( cms ? 'cms' : 'local' )).then( this.registryWritten.bind(this) ).catch( this.promiseErrorOutputsCMS.bind(this) );
    this.classes.registry.write('embedinfo', 'outputsoverride', JSON.stringify(outputs)).then( this.registryWritten.bind(this) ).catch( this.promiseErrorOutputs.bind(this) );
    

    if( !setup )
    {
        this.detectScreen();

        this.setScreenConfig();
    }
};

BrightSign.prototype.overwriteResolution = function( resolution, setup )
{

    this.resolutionOverride = resolution;

    if( this.registryData && !this.registryData.embedinfo )
    {
        this.registryData.embedinfo = {};
    }

    this.registryData.embedinfo.resolutionoverride = this.resolutionOverride;

    this.classes.registry.write('embedinfo', 'resolutionoverride', resolution).then( this.registryWritten.bind(this) ).catch( this.promiseErrorResolution.bind(this) );
    
    var res = resolution.split('x');

    this.classes.registry.write('embedinfo', 'resolutionwidth', res[0]).then( this.registryWritten.bind(this) ).catch( this.promiseErrorResolutionWidth.bind(this) );
    this.classes.registry.write('embedinfo', 'resolutionheight', res[1]).then( this.registryWritten.bind(this) ).catch( this.promiseErrorResolutionHeight.bind(this) );

    if( !setup )
    {
        // this.detectScreen();
        this.updatePreferredMode();

        this.setScreenConfig();
    }
};

BrightSign.prototype.updateAudioOutput = function( audioOutput )
{

    this.audioOutput = audioOutput;

    if( this.registryData && !this.registryData.embedinfo )
    {
        this.registryData.embedinfo = {};
    }

    this.registryData.embedinfo.audiooutput = this.audioOutput;

    this.classes.registry.write('embedinfo', 'audiooutput', audioOutput).then( this.registryWritten.bind(this) ).catch( this.promiseErrorAudio.bind(this) );

};

BrightSign.prototype.detectAudio = function()
{

    if( this.registryData && this.registryData.embedinfo && this.registryData.embedinfo.audiooutput )
    {
        this.audioOutput = this.registryData.embedinfo.audiooutput;
    }
    else
    {
        this.audioOutput = 'hdmi';
    }

};

BrightSign.prototype.detectScreen = function()
{

    if( this.registryData && this.registryData.embedinfo && this.registryData.embedinfo.orientation )
    {

        this.orientation = this.registryData.embedinfo.orientation;

    }
    else
    {

        this.orientation = 'landscape';

    }

    if( this.registryData && this.registryData.embedinfo && this.registryData.embedinfo.resolutionoverride )
    {

        this.resolutionOverride = this.registryData.embedinfo.resolutionoverride;

    }
    else
    {

        this.resolutionOverride = false;

    }

    // New multi output changes.
    // getScreenModes
    // setScreenModes

    if( this.multiOutputModels.indexOf(app.device.details.model) != -1 )
    {
        this.classes.videoModeConfig.getScreenModes().then(this.screenModesResult.bind(this)).catch(this.promiseErrorGetScreenModes.bind(this));
    }

	this.classes.videoModeConfig.getBestMode('hdmi').then( this.bestModeResult.bind(this) ).catch( this.promiseErrorGetBestMode.bind(this) );
    this.classes.videoModeConfig.getAvailableModes().then( this.availableModesResult.bind(this) ).catch( this.promiseErrorAvailableModes.bind(this) );

};

BrightSign.prototype.screenModesResult = function(data)
{

    this.multipleOutputs = data;

    app.device.details.stats.multiOutputs = this.multipleOutputs;
    app.device.details.stats.multiOutputsCMS = this.registryData.embedinfo.outputsoverridecms;

};

// New multi output changes.
BrightSign.prototype.setScreenModes = function()
{

    if( !this.outputsOverride && this.multipleOutputs )
    {
        this.outputsOverride = this.multipleOutputs;
    }
    
    if( this.outputsOverride )
    {
        
        var maxHeight = 0;
        var maxWidth = 0;

        for( var i = 0; i < this.outputsOverride.length; i++ )
        {
            if( [false, 'false'].indexOf(this.outputsOverride[i].enabled) != -1 )
            {

                if( this.multipleOutputs[i] )
                {
                    this.multipleOutputs[i].enabled = false;
                }
            }
            else
            {
                if( this.multipleOutputs[i] )
                {
                    this.multipleOutputs[i].enabled = true;
                }

                if( ['normal', '180'].indexOf(this.outputsOverride[i].transform) != -1 )
                {
                    if( this.outputsOverride[i].videoMode && parseInt(this.outputsOverride[i].videoMode.split('x')[0]) + parseInt(this.outputsOverride[i].screenX) > maxWidth )
                    {
                        maxWidth = parseInt(this.outputsOverride[i].videoMode.split('x')[0]) + parseInt(this.outputsOverride[i].screenX);
                    }

                    if( this.outputsOverride[i].videoMode && parseInt(this.outputsOverride[i].videoMode.split('x')[1]) + parseInt(this.outputsOverride[i].screenY) > maxHeight )
                    {
                        maxHeight = parseInt(this.outputsOverride[i].videoMode.split('x')[1]) + parseInt(this.outputsOverride[i].screenY);
                    }
                }
                else if( ['90', '270'].indexOf(this.outputsOverride[i].transform) != -1 )
                {
                    if( this.outputsOverride[i].videoMode && parseInt(this.outputsOverride[i].videoMode.split('x')[1]) + parseInt(this.outputsOverride[i].screenX) > maxWidth )
                    {
                        maxWidth = parseInt(this.outputsOverride[i].videoMode.split('x')[1]) + parseInt(this.outputsOverride[i].screenX);
                    }

                    if( this.outputsOverride[i].videoMode && parseInt(this.outputsOverride[i].videoMode.split('x')[0]) + parseInt(this.outputsOverride[i].screenY) > maxHeight )
                    {
                        maxHeight = parseInt(this.outputsOverride[i].videoMode.split('x')[0]) + parseInt(this.outputsOverride[i].screenY);
                    }
                }
                // this.outputsOverride[i].enabled = true;
            }

            if( this.multipleOutputs[i] )
            {
                this.multipleOutputs[i].videoMode = this.outputsOverride[i].videoMode + ':fullres:gfxmemlarge'; // gfxmemdefault:';
                this.multipleOutputs[i].transform = this.outputsOverride[i].transform;
                this.multipleOutputs[i].screenX = parseInt(this.outputsOverride[i].screenX);
                this.multipleOutputs[i].screenY = parseInt(this.outputsOverride[i].screenY);
            }
            // this.multipleOutputs[i].screenX = this.outputsOverride[i].screenX;
        }

        // Should have to set this
        // if( this.orientation == 'portrait' || this.orientation == 'reversedPortrait' )
        // {
        //     document.body.style.width = maxHeight + 'px';
        //     document.body.style.height = maxWidth + 'px';   
        // }
        // else
        // {
        //     document.body.style.width = maxWidth + 'px';
        //     document.body.style.height = maxHeight + 'px';
        // }

        this.classes.videoModeConfig.setScreenModes(this.multipleOutputs).then( this.checkReboot.bind(this) ).catch( this.promiseErrorSetScreenModes.bind(this) );

    }
    else
    {
        var maxHeight = 0;
        var maxWidth = 0;

        for( var i = 0; i < this.multipleOutputs.length; i++ )
        {
            if( this.multipleOutputs[i].enabled == 'true' || this.multipleOutputs[i].enabled === true )
            {
                if( this.multipleOutputs[i].videoMode && parseInt(this.multipleOutputs[i].videoMode.split('x')[0]) + parseInt(this.multipleOutputs[i].screenX) > maxWidth )
                {
                    maxWidth = parseInt(this.multipleOutputs[i].videoMode.split('x')[0]) + parseInt(this.multipleOutputs[i].screenX);
                }

                if( this.multipleOutputs[i].videoMode && parseInt(this.multipleOutputs[i].videoMode.split('x')[1]) + parseInt(this.multipleOutputs[i].screenY) > maxHeight )
                {
                    maxHeight = parseInt(this.multipleOutputs[i].videoMode.split('x')[1]) + parseInt(this.multipleOutputs[i].screenY);
                }

            }
        }

        // Should have to set this
        // if( this.orientation == 'portrait' || this.orientation == 'reversedPortrait' )
        // {
        //     document.body.style.width = maxHeight + 'px';
        //     document.body.style.height = maxWidth + 'px';   
        // }
        // else
        // {
        //     document.body.style.width = maxWidth + 'px';
        //     document.body.style.height = maxHeight + 'px';
        // }
    }
     
    

};

BrightSign.prototype.hdmiChanged = function(evt)
{

    // console.log(evt);

    // this.detectScreen();

};

// New multi output changes.
BrightSign.prototype.bestModeResult = function( data )
{

	if( data != '' && data != 'Cannot read edid information.' )
    {

        if( this.availableOutputs.indexOf('hdmi') != -1 )
        {
            this.availableOutputs.push('hdmi');
        }
        
        this.bestScreenModeName = data;

        // this.setMode( this.bestHDMIMode );

    }
    else
    {
    	this.bestScreenModeName = '1920x1080x60p'; 
    }

};

// New multi output changes.
BrightSign.prototype.availableModesResult = function( data )
{

    this.processAvailableModes( data );

    

};

BrightSign.prototype.setScreenConfig = function()
{

    if( this.multiOutputModels.indexOf(app.device.details.model) != -1 )
    {
        this.setScreenModes();
    }
    else
    {
        this.setScreenMode();
    }

};

// New multi output changes.
BrightSign.prototype.processAvailableModes = function( data )
{

    this.availableScreenModes = data;

    // app.device.details.stats.availableRes = JSON.stringify(this.availableScreenModes);

    // check for override
    // 

    // console.log(this.resolutionOverride);

    for( var i = 0; i < this.availableScreenModes.length; i++ )
    {

        if( this.resolutionOverride && this.availableScreenModes[i].modeName == this.resolutionOverride )
        {

            this.preferredScreenMode = this.availableScreenModes[i];

        }
        else if( !this.resolutionOverride && ( this.availableScreenModes[i].preferred || this.availableScreenModes[i].modeName == this.bestScreenModeName ) )
        {

            this.preferredScreenMode = this.availableScreenModes[i];

        }

    }

    app.device.details.stats.preferredRes = this.preferredScreenMode;

    this.fullResolutionHack();

}

BrightSign.prototype.updatePreferredMode = function()
{
    for( var i = 0; i < this.availableScreenModes.length; i++ )
    {

        if( this.resolutionOverride && this.availableScreenModes[i].modeName == this.resolutionOverride )
        {

            this.preferredScreenMode = this.availableScreenModes[i];

        }
        // else if( !this.resolutionOverride && ( this.availableScreenModes[i].preferred || this.availableScreenModes[i].modeName == this.bestScreenModeName ) )
        // {

        //     this.preferredScreenMode = this.availableScreenModes[i];

        // }

    }

    app.device.details.stats.preferredRes = this.preferredScreenMode;

    this.fullResolutionHack();
};

// New multi output changes.
BrightSign.prototype.setScreenMode = function()
{

    var configData = {};

    // https://brightsign.atlassian.net/wiki/spaces/DOC/pages/370678836/Full-Resolution+Graphics
    if( this.checkFullRes() )
    {
        configData.modeName = this.preferredScreenMode.modeName + ':fullres:gfxmemlarge';
    }
    else
    {
        configData.modeName = this.preferredScreenMode.modeName;
    }
    
    configData.overscan = false;

    // Think may be able to enable full 4k somehow using below
    // configData.fullres = true;
    // configData.gfxmemlarge = true;
    
    // this.classes.registry.write('embedInfo', 'Orientation', 'landscape').then( this.registryWritten.bind(this) ).catch( this.promiseError.bind(this) );

    // console.log('set screen mode: ' + configData.modeName);
    this.fullResolutionHack();
    
    if( this.families.s5.indexOf(app.device.details.family) != -1 )
    {
        var rotation = 'normal';
        switch(this.orientation)
        {
            case 'portrait' :
                rotation = '90';
            break;

            case 'reversedPortrait' :
                rotation = '270';
            break;

            case 'reversedLandscape' :
                rotation = '180';
            break;
        }
        this.classes.videoModeConfig.setScreenModes([{enabled: true, outputName: 'HDMI-1', screenX: 0, screenY: 0, transform: rotation, videoMode: configData.modeName}]).then( this.checkReboot.bind(this) ).catch( this.promiseErrorSetMode.bind(this) );
    }
    else
    {
        this.classes.videoModeConfig.setMode(configData).then( this.checkReboot.bind(this) ).catch( this.promiseErrorSetMode.bind(this) );
    }

};

BrightSign.prototype.fullResolutionHack = function()
{
    

    // will cause issues on xc.
    if( !this.checkFullRes() && ( this.preferredScreenMode.modeName.indexOf('4096x2160') != -1 || this.preferredScreenMode.modeName.indexOf('3840x2160') != -1 ) )
    {
        var res = this.preferredScreenMode.modeName.split('x');

        if( this.orientation == 'portrait' || this.orientation == 'reversedPortrait' )
        {
            document.body.style.width = res[1] + 'px';
            document.body.style.height = res[0] + 'px';   
        }
        else
        {
            document.body.style.width = res[0] + 'px';
            document.body.style.height = res[1] + 'px';
        }

        document.body.style.transform = 'scale(0.5)';
        document.body.style.transformOrigin = '0 0';

    }
};

// New multi output changes.
BrightSign.prototype.setScreenOrientation = function( newOrientation )
{

    if( !newOrientation )
    {
        newOrientation = 'landscape';
    }

    if( this.orientation != newOrientation )
    {
        this.classes.registry.write('embedinfo', 'orientation', newOrientation).then( this.orientationChanged.bind(this) ).catch( this.promiseErrorSetOrientation.bind(this) );
    }   

};

// New multi output changes.
BrightSign.prototype.orientationChanged = function()
{

    this.reboot();

};

// Network
BrightSign.prototype.detectNetwork = function()
{

    this.currentNetworkConfig = this.registryData.embednetwork;

	var interfaces = this.classes.os.networkInterfaces();

    var DNS = this.classes.dns.getServers();

    if( this.currentNetworkConfig && this.currentNetworkConfig.networktype == 'manual' )
    {
        DNS = this.currentNetworkConfig.dns;
    }
    else
    {
        DNS = [];
    }

	this.availableInterfaces = [];

	if( interfaces.eth0 )
	{

		for( var i = 0; i < interfaces.eth0.length; i++ )
		{

			if( !interfaces.eth0[i].internal && interfaces.eth0[i].family == 'IPv4' )
			{

				interfaces.eth0[i].interface = 'ethernet';
                interfaces.eth0[i].dns = DNS;

				this.availableInterfaces.push(
					interfaces.eth0[i]
				);

			}

		}

	}
    else if( this.currentNetworkConfig && this.currentNetworkConfig.networkinterface == 'ethernet' )
    {

        // setTimeout(this.detectNetwork.bind(this), 3000);

    }

	if( interfaces.wlan0 )
	{

		for( var i = 0; i < interfaces.wlan0.length; i++ )
		{

			if( !interfaces.wlan0[i].internal && interfaces.wlan0[i].family == 'IPv4' )
			{

				interfaces.wlan0[i].interface = 'wifi';
                interfaces.wlan0[i].dns = DNS;
				
				this.availableInterfaces.push(
					interfaces.wlan0[i]
				);

			}

		}

	}
    else if( this.currentNetworkConfig && this.currentNetworkConfig.networkinterface == 'wifi' )
    {

        // setTimeout(this.detectNetwork.bind(this), 3000);

    }

    this.updateDeviceStats();
};

BrightSign.prototype.updateDeviceStats = function()
{
    if( this.storage )
    {
        app.device.details.stats.storageSpace = app.file.formatBytes(this.storage.free, 1);
        app.device.details.stats.totalStorageSpace = app.file.formatBytes(this.storage.total, 1);
    }


    // need to do same as other, var 

    var availableInterface = false;


    for( var i = 0; i < this.availableInterfaces.length; i++ )
    {

        if( this.currentNetworkConfig && this.currentNetworkConfig.networkinterface == this.availableInterfaces[i].interface )
        {
            availableInterface = this.availableInterfaces[i];
        }

    }

    if( !availableInterface && this.availableInterfaces.length )
    {
        availableInterface = this.availableInterfaces[0];
    }

    if( availableInterface )
    {
        app.device.details.stats.macAddress = availableInterface.mac;
        app.device.details.stats.ipAddress    = availableInterface.address;
        app.device.details.stats.subnetMask = availableInterface.netmask;


        if( availableInterface.dns && availableInterface.dns.length )
        {

            var dnsValue = '';

            for( var e = 0; e < availableInterface.dns.length; e++ )
            {

                if( e == 0 )
                {
                    dnsValue = dnsValue += availableInterface.dns[e];
                }
                else
                {
                    dnsValue = dnsValue += ',' + availableInterface.dns[e];  
                }

            }

            app.device.details.stats.dns = dnsValue;

        }
        else if( this.currentNetworkConfig && this.currentNetworkConfig.dns1 )
        {

            app.device.details.stats.dns = this.currentNetworkConfig.dns1;
            
        }
        else
        {
            app.device.details.stats.dns = '';
        }

        app.device.details.connection = availableInterface.interface;
    }

    if( this.ram )
    {

        var percent     = ( ( this.ram.free / this.ram.total ) * 100 ).toFixed(2);
        var available   = app.file.formatBytes( this.ram.free, 2 );
        var total       = app.file.formatBytes( this.ram.total, 2 );

        app.device.details.stats.ram = available + ' / ' + total + ' ' + '(' + percent + '%)';

    }

    if( this.cpu )
    {

        app.device.details.stats.cpu = this.cpu.percent + '% (' + this.cpu.count + ' CPUs)';

    }
};

BrightSign.prototype.updateNetwork = function( details, setup )
{

    if( !details )
    {   

        var details = {
            'networkInterface' : 'ethernet',
            'ethernet' : {
                'type' : 'dhcp'
            },
            'tcpip' : {
                'type' : 'dhcp'
            }
        };

    }


    // console.log(details);

    if( details[ details.networkInterface ] && details[ details.networkInterface ].type == 'manual' )
    {

        var dns = details[ details.networkInterface ].dns.split(',');

        var config = {
            'networkinterface' : details.networkInterface,
            'networktype'   : details[ details.networkInterface ].type,
            'ipaddress'     : details[ details.networkInterface ].ipAddress,
            'ipaddressold'     : this.availableInterfaces[0].address,
            'subnetmask'    : details[ details.networkInterface ].subnetMask,
            'gateway'       : details[ details.networkInterface ].defaultGateway,
            'broadcast'     : details[ details.networkInterface ].broadcast
        };

        if( dns[0] )
        {
            config.dns1 = dns[0];
        }
        else
        {
            config.dns1 = null;
        }

        if( dns[1] )
        {
            config.dns2 = dns[1];
        }
        else
        {
            config.dns2 = null;
        }

        if( dns[2] )
        {
            config.dns3 = dns[2];
        }
        else
        {
            config.dns3 = null;
        }

        
    }
    else if( details.tcpip.type == 'manual' )
    {
        var dns = details.tcpip.dns.split(',');

        var availableInterface = false;

        for( var i = 0; i < this.availableInterfaces.length; i++ )
        {

            if( this.availableInterfaces[i].interface == details.networkInterface )
            {
                availableInterface = this.availableInterfaces[i];
            }

        }

        if( !availableInterface && this.availableInterfaces[0] )
        {
            availableInterface = this.availableInterfaces[0];
        }

        var config = {
            'networkinterface' : details.networkInterface,
            'networktype'   : details.tcpip.type,
            'ipaddress'     : details.tcpip.ipAddress,
            'ipaddressold'     : availableInterface.address,
            'subnetmask'    : details.tcpip.subnetMask,
            'gateway'       : details.tcpip.defaultGateway,
            'broadcast'     : details.tcpip.broadcast
        };

        if( dns[0] )
        {
            config.dns1 = dns[0];
        }
        else
        {
            config.dns1 = null;
        }

        if( dns[1] )
        {
            config.dns2 = dns[1];
        }
        else
        {
            config.dns2 = null;
        }

        if( dns[2] )
        {
            config.dns3 = dns[2];
        }
        else
        {
            config.dns3 = null;
        }
    }
    else
    {
        var config = {
            'networkinterface'  : details.networkInterface,
            'networktype'       : ( details.tcpip && details.tcpip.type ? details.tcpip.type : details[ details.networkInterface ].type ),
            // 'additional'       : details[ details.networkInterface ]
        };
       
        config.dns1 = null;
        config.dns2 = null;
        config.dns3 = null;

        config.gateway = null;
        config.broadcast = null;
        config.subnetmask = null;
        config.ipaddress = null;

    }

    buffer = require('buffer').Buffer;

    if( config.networkinterface == 'wifi' )
    {

        



        config.wifiSecurity = details[ details.networkInterface ].security;
        config.wifiSSID = details[ details.networkInterface ].esid;

        var password = buffer.from(details[ details.networkInterface ].password);

        config.wifiPassword = password.toString('base64');

    }
    else
    {
        config.wifiSecurity = null;
        config.wifiSSID = null;
        config.wifiPassword = null;
    }

    if( details.proxy && details.proxy.enable == 'yes' )
    {

        var splitAddress = details.proxy.address.split('://');

        if( splitAddress[1] )
        {
            var protocol = splitAddress[0] + '://';
        }
        else
        {
            var protocol = 'http://';
        }

        var proxystring = protocol + details.proxy.username + ':' + details.proxy.password + '@' + splitAddress[1] + ':' + details.proxy.port;

        config.useproxy     = 'yes';
        config.proxystring  = proxystring;
        config.proxyaddress = details.proxy.address;
        config.proxyport    = details.proxy.port;
        config.proxyuser    = details.proxy.username;
        config.proxypass    = details.proxy.password;

    }
    else
    {

        config.useproxy     = 'no';

        config.proxyaddress = null;
        config.proxyport    = null;
        config.proxyuser    = null;
        config.proxypass    = null;
        config.proxystring  = null;

    }

    // console.log(config);

    config.networkupdates = 'yes';

    if( setup )
    {
        this.classes.registry.write('embednetwork', config).then( this.networkSetupUpdated.bind(this) ).catch( this.promiseError.bind(this) );
    }
    else
    {
        this.classes.registry.write('embednetwork', config).then( this.networkUpdated.bind(this) ).catch( this.promiseErrorNetwork.bind(this) );
    }
    // this.classes.registry.write('embedNetwork', 'networkType', details[ details.networkInterface ].type)


    // 0AU0HSHK400542t

    // if( details.networkInterface == 'ethernet' )
    // {

        

    // }
    // else if( details.networkInterface == 'wifi' )
    // {



    // }

    // var config    = this.classes.network.ethernet.defaultConfig();

    // // this.classes.network.ethernet.getConfig().then( function( config ) {

    //     if( newConfig.dhcp == 'no' )
    //     {

    //         config.dnsServerList = [newConfig.dns];
    //         config.ipAddressList = [
    //             {
    //                 'family' : 'IPv4',
    //                 'address': newConfig.ipAddress,
    //                 'netmask': newConfig.subnetMask
    //                 // 'gateway': newConfig.defaultGateway,
    //                 // 'broadcast' : newConfig.ipAddress
    //             }
    //         ];

    //     }

    //     console.log(config);

    // this.classes.network.ethernet.applyConfig( config ).then( 
    //     function( response ) {
    //         console.log(response);

    //         // this.reboot();
    //     }.bind(this)
    // ).catch( 
    //     function(e) {
    //         console.log(e);
    //     } 
    // );

};

BrightSign.prototype.networkUpdated = function()
{

    // console.log('network updated');
    setTimeout( this.reboot.bind(this), 2000 );

};

BrightSign.prototype.networkSetupUpdated = function()
{

    // console.log('setup network complete');

};


BrightSign.prototype.detectCPU = function()
{

	// this.cpus = this.classes.os.cpus();
	this.loadAvg = this.classes.os.loadavg();

	var startCPU = this.trackCPU();

	var x = 0;

	var date = new Date().getTime();
	var newDate = new Date().getTime();

	while( ( newDate - date ) < 3000 )
	{
		
		newDate = new Date().getTime();

	}

	var endCPU = this.trackCPU();

	var diff = {
		'idle' : endCPU.idle  - startCPU.idle,
		'total': endCPU.total - startCPU.total,
		'count'	: startCPU.count
	};

	diff.percent = ( ( 1 - diff.idle / diff.total ) * 100 ).toFixed(2);

	this.cpu = diff;

};

BrightSign.prototype.trackCPU = function()
{

	var cpus = this.classes.os.cpus();

	var totalIdle = 0;
    var total = 0;
    var numOfProcessors = 0;

    for( var i = 0; i < cpus.length; i++ )
    {

    	numOfProcessors++;

        totalIdle   = totalIdle + cpus[i].times.idle;
        total       = total + ( cpus[i].times.user + cpus[i].times.nice + cpus[i].times.sys + cpus[i].times.irq + cpus[i].times.idle );

    }

    return {
    	'idle' : ( totalIdle / numOfProcessors ),
    	'total' : ( total / numOfProcessors ),
    	'count' : numOfProcessors
    };

};

BrightSign.prototype.detectRAM = function()
{

	this.ram = {
		'total' : this.classes.os.totalmem(),
		'free'	: this.classes.os.freemem()
	}

};

BrightSign.prototype.detectStorage = function()
{

    var path = '/storage/sd';

    this.classes.disk.check(path, this.storageResult.bind(this));

};

BrightSign.prototype.storageResult = function(err, info)
{

    if( err )
    {
        this.storage = {
            'total' : 0,
            'free'  : 0,
            'available' : 0
        };

        // console.log(err);
    }
    else
    {

        this.storage = {
            'total' : info.total,
            'free'  : info.free,
            'used' : info.used
        };

    }

};

BrightSign.prototype.registryWritten = function()
{



};

BrightSign.prototype.checkReboot = function( response )
{

    if( response.restartRequired === true ) 
    {
        this.reboot();
    }

};

BrightSign.prototype.setDate = function(dateString)
{
    
    var date = new Date( dateString );

    this.classes.time.setDate( date );

};

BrightSign.prototype.setTimezone = function( newTZ ) 
{
    
    if( this.localTimeOffset != newTZ )
    {

        // console.log('setting timezone');

        if( newTZ == 0 || !newTZ )
        {
            newTZ = '';
        }
        else if( newTZ.indexOf('-') == -1 )
        {
            newTZ = '+' + newTZ; 
        }

        this.classes.time.setTimeZone('GMT' + newTZ).then( function(evt) {


            this.reboot();
            // console.log(newTZ, this.localTimeOffset);
        }.bind(this) );

    }

};

BrightSign.prototype.reboot = function()
{

    // Was caused by setOrientation & set timezone.
    if( app.device.savingData )
    {
        // to avoid truncating cmsData.txt

        setTimeout(this.reboot.bind(this), 1000);
        return;
    }

	this.classes.system.reboot();

};

BrightSign.prototype.promiseError = function( data )
{

    console.log('promise error: ' + JSON.stringify(data));

};

BrightSign.prototype.promiseErrorConfig = function( data )
{

    console.log('promise error config: ' + JSON.stringify(data));

};

BrightSign.prototype.promiseErrorAuth = function( data )
{

    console.log('promise error auth: ' + JSON.stringify(data));

};

BrightSign.prototype.promiseErrorOutputs = function( data )
{

    console.log('promise error outputs: ' + JSON.stringify(data));

};

BrightSign.prototype.promiseErrorOutputsCMS = function( data )
{

    console.log('promise error outputs cms: ' + JSON.stringify(data));

};

BrightSign.prototype.promiseErrorResolution = function( data )
{

    console.log('promise error resolution: ' + JSON.stringify(data));

};

BrightSign.prototype.promiseErrorAudio = function( data )
{

    console.log('promise error audio: ' + JSON.stringify(data));

};

BrightSign.prototype.promiseErrorGetScreenModes = function( data )
{

    console.log('promise error promiseErrorGetScreenModes: ' + JSON.stringify(data));

};
BrightSign.prototype.promiseErrorGetScreenModes = function( data )
{

    console.log('promise error promiseErrorGetScreenModes: ' + JSON.stringify(data));

};

BrightSign.prototype.promiseErrorGetBestMode = function( data )
{

    console.log('promise error promiseErrorGetBestMode: ' + JSON.stringify(data));

};
BrightSign.prototype.promiseErrorAvailableModes = function( data )
{

    console.log('promise error promiseErrorAvailableModes: ' + JSON.stringify(data));

};

BrightSign.prototype.promiseErrorSetScreenModes = function( data )
{

    console.log('promise error promiseErrorSetScreenModes: ' + JSON.stringify(data));

};


BrightSign.prototype.promiseErrorResolutionWidth = function( data )
{

    console.log('promise error promiseErrorResolutionWidth: ' + JSON.stringify(data));

};

BrightSign.prototype.promiseErrorResolutionHeight = function( data )
{

    console.log('promise error promiseErrorResolutionHeight: ' + JSON.stringify(data));

};

BrightSign.prototype.promiseErrorSetOrientation = function( data )
{

    console.log('promise error promiseErrorResolutionHeight: ' + JSON.stringify(data));

};


BrightSign.prototype.promiseErrorSetMode = function( data )
{

    console.log('promise error promiseErrorSetMode: ' + JSON.stringify(data));

};

BrightSign.prototype.promiseErrorNetwork = function( data )
{

    console.log('promise error promiseErrorNetwork: ' + JSON.stringify(data));

};




BrightSign.prototype.rs232 = function()
{

    



};



















