// ----------------------
// -- Channel Manager ---
// ----------------------

var EmbedChannel = function() {

    return this;

};

EmbedChannel.prototype.info = {
    'md5'       : '',
    'fileList'  : [],
    'schedules' : [],
    'overrides' : []
};

EmbedChannel.prototype.save = function() 
{
    
};

EmbedChannel.prototype.download = function() 
{

    if( this.cancelDownload )
    {

        app.file.emptyDirectory('tmp', function() {

            this.cancelDownload = false;

            this.resetDownload();

        }.bind(this));

        return;

    }
    
    if( !this.filesToDownload[ this.curCount ] )
    {

        this.afterDownload();

    }
    else
    {

        app.file.downloadFile(this.filesToDownload[ this.curCount ].url, 'tmp', this.filesToDownload[ this.curCount ].slug, function( success, error ) {

            if( success )
            {
                this.curCount++;
            }
            else
            {
                if( !this.currentDownloadAttempts )
                {
                    this.currentDownloadAttempts = 0;
                }

                if( this.currentDownloadAttempts == 3 )
                {
                    
                    this.cancelDownload = true;

                    app.notifications.send({
                        'type' : 'error',
                        'title' : 'Could not download file',
                        'message' : 'Failed to download file ' + this.filesToDownload[this.curCount].slug + ', code: ' + error.code
                    });

                    app.messages.show('downloadFailed');

                }
                else
                {
                    this.currentDownloadAttempts++;
                }

            }
            
            this.download();

        }.bind(this), this.onProgress.bind(this), this.filesToDownload[ this.curCount ].size, this.filesToDownload[ this.curCount ].path );

    }

};

EmbedChannel.prototype.onProgress = function(progressEvent) 
{   
    if( typeof progressEvent != 'string' && progressEvent.lengthComputable && !this.cancelDownload )
    {

        var percent = ( ( progressEvent.loaded / progressEvent.total ) * 100 ).toFixed(0) + '%';

        app.messages.domEl.belowStatus.querySelectorAll('p')[0].innerHTML = '<span class="percent">' + percent + '</span> <span class="fileCount">' + ( this.curCount + 1 ) + '/' + this.filesToDownload.length + '</span><br><span>' + this.filesToDownload[ this.curCount ].slug + ' (' + app.file.formatBytes( this.filesToDownload[ this.curCount ].size, 1 ) + ')</span>';
        
    }
};

EmbedChannel.prototype.afterDownload = function()
{

    this.deleteOldFiles( 0, function() {

        this.moveNewFiles( 0, function() {

            app.file.writeFile( 'info', 'fileList.txt', JSON.stringify( this.newFileList ), function() {

                app.device.CMSData.check                = this.newCheck;

                app.device.details.stats.channelReceived = new Date().toString();
                
                app.device.saveCMSData();

                this.checkSchedule( true );

                this.resetDownload();

                app.brightSign.detectStorage();

            }.bind(this) );

        }.bind(this) );

    }.bind(this) );

};

EmbedChannel.prototype.resetDownload = function()
{

    this.filesThatMatch     = null;
    this.filesToDelete      = null;
    this.filesToDownload    = null;
    this.newFileList        = null;
    this.newCheck           = null;

    this.maxCount           = null;
    this.curCount           = 0;

    this.isDownloading      = null;

};

EmbedChannel.prototype.deleteOldFiles = function( index, callback )
{

    if( this.filesToDelete[ index ] )
    {
        app.file.deleteFile( 'files', this.filesToDelete[ index ].slug, function( success ) {

            this.deleteOldFiles( index + 1, callback );

        }.bind(this), this.filesToDelete[ index ].path );
    }
    else
    {
        callback();
    }

};

EmbedChannel.prototype.moveNewFiles = function( index, callback )
{

    if( this.filesToDownload[ index ] )
    {   

        app.file.moveFile( 'tmp', this.filesToDownload[ index ].slug, 'files', this.filesToDownload[ index ].slug, function( success ) {

            this.moveNewFiles( index + 1, callback );
            
        }.bind(this), this.filesToDownload[ index ].path );
    }
    else
    {
        callback();
    }

};

EmbedChannel.prototype.downloadSuccess = function(success, url) 
{

};

EmbedChannel.prototype.clear = function( callback )
{

    if( this.isDownloading )
    {
        this.cancelDownload = true;
    }

    app.file.emptyDirectory('tmp', function( success ) {
    });

    app.file.emptyDirectory('files', function( success ) {
    });

    if( app.device.CMSData )
    {
        app.device.CMSData.check = false;
        app.device.CMSData.channel = false;
    }
    
    app.device.saveCMSData();

    app.file.deleteFile('info', 'fileList.txt', function(success) {
            
        if( success )
        {
            app.brightSign.detectStorage();
        }

        if( callback )
        {
            callback();
        }

    });

    app.layout.show(false);

};



EmbedChannel.prototype.checkMD5 = function( md5 ) 
{   

    if( app.device.CMSData.check != md5 )
    {
        return true;
    }
    else
    {
        return false;
    }

};

EmbedChannel.prototype.update = function( files, md5 )
{

    // get current file list,
    // check current file list for matches with new.
    // add unused items to array for deletion
    // 
    // download new files into tmp directory.
    // delete unused items from files directory
    // move tmp files into files directory.

    // reload playback.
    // update channel received, storage etc
    // update channel check and file list.

    if( !this.isDownloading )
    {
    
        // Ping new api to get signed urls.
        this.fetchFileList(md5, files, function(md5, oldFiles, files) {
            this.isDownloading  = true;
            this.newFileList    = files;
            this.newCheck       = md5;

            app.file.read( 'info', 'fileList.txt', this.checkFileMD5s.bind(this), true );
        }.bind(this));

    }

};

EmbedChannel.prototype.fetchFileList = function(md5, files, callback )
{

    var request = new Request({
        'url'   : app.config.storageUrl + '/api/v2/' + app.device.accountKey  + '/channel/files',
        'type'  : 'POST',
        'data'	: {
        	'embedID' : app.device.details.embedID
        }
    });

    request.onSuccess = function(newFiles) {
        callback(md5, files, newFiles);
    };

    request.onError = function() {
        callback(md5, files, files);
    };

};

EmbedChannel.prototype.checkFileMD5s = function( success, content, sPath )
{

    var localFiles = [];

    if( success && content && content != '' && content != '[]' )
    {
        localFiles = JSON.parse( content );
    }

    // console.log(localFiles, this.newFileList);

    var filesToDelete   = [];
    var filesThatMatch  = [];
    var filesToDownload = [];

    var totalSize       = 0;

    for( var i = 0; i < this.newFileList.length; i++ )
    {

        var match = this.match( this.newFileList[i], localFiles );

        if( match )
        {
            filesThatMatch.push( this.newFileList[i] );
        }
        else
        {
            filesToDownload.push( this.newFileList[i] );
            totalSize = totalSize + parseInt( this.newFileList[i].size );
            // filesToDelete.push( localFiles[i] ); 
        }

    }

    for( var i = 0; i < localFiles.length; i++ )
    {

        var match = this.match( localFiles[i], filesThatMatch );

        if( !match )
        {
            filesToDelete.push( localFiles[i] );
        }

    }   


    this.filesToDelete      = filesToDelete;
    this.filesToDownload    = filesToDownload;
    this.filesThatMatch     = filesThatMatch;

    // Check available space compared with files to download
    // If enough, start download

    if( app.brightSign.storage.free > totalSize )
    {

        app.messages.show( 'downloading' );

        this.startDownload();

    }
    else
    {
        // Else
        var notification = {
            'type'  : 'error',
            'title' : 'Channel too big',
            'body'  : 'The channel you have published is too big you need [ ' + app.file.formatBytes((totalSize), 1) + ' ] to download this channel. You only have [ ' + app.file.formatBytes(app.brightSign.storage.free, 1) + ' ] available . Please select an alternative storage option or modify your channel accordingly.',
        };

        app.notifications.send(notification);

        app.messages.messages.channelTooBig.description = notification.body;

        app.messages.show('channelTooBig');

        // Send notification not enough storage.
        this.resetDownload();
    }

};

EmbedChannel.prototype.startDownload = function()
{

    this.maxCount = this.filesToDownload.length;
    this.curCount = 0;

    app.file.emptyDirectory('tmp', function( success ) {
        this.download();
    }.bind(this));

};

EmbedChannel.prototype.match = function( fileObj, list )
{

    var match = false;

    for( var i = 0; i < list.length; i++ )
    {

        if( list[i].md5 == fileObj.md5 && list[i].slug == fileObj.slug )
        {
            if( fileObj.path )
            {
                if( list[i].path == fileObj.path )
                {
                    match = true;
                }
            }
            else
            {
                match = true;
            }

        }

    }
    
    return match;

};

EmbedChannel.prototype.checkSchedule = function( force )
{

    if( !app.device.CMSData.check || !app.device.CMSData.channel || !app.device.CMSData.channel.schedules || !app.device.CMSData.channel.overrides )
    {
        app.layout.show(false, force);
        return;
    }

    var schedules = app.device.CMSData.channel.schedules;
    var overrides = app.device.CMSData.channel.overrides;

    var current_date_time   = new Date();
    var timestamp           = current_date_time.getTime();

    var layoutName = '';
    var screenOff = false;

    if( schedules ) 
    {
        for( var i = 0; i < schedules.length; i++ )
        {
            var val = schedules[i];

            var layout  = val.layout_id;
            var from    = val.start_time;
                from    = from.split(':');
            var to      = val.end_time;
                to      = to.split(':');
            var days    = JSON.parse( val.days );

            var year    = current_date_time.getFullYear();
            var month   = current_date_time.getMonth();
            var day     = current_date_time.getDate();
            var dayname = current_date_time.getDay();
            var hour    = current_date_time.getHours();
            var minute  = current_date_time.getMinutes();
            var second  = current_date_time.getSeconds();

            if( dayname == 0 ) {
                dayname = 'sun';
            } else if( dayname == 1 ) {
                dayname = 'mon';
            } else if( dayname == 2 ) {
                dayname = 'tue';
            } else if( dayname == 3 ) {
                dayname = 'wed';
            } else if( dayname == 4 ) {
                dayname = 'thu';
            } else if( dayname == 5 ) {
                dayname = 'fri';
            } else if( dayname == 6 ) {
                dayname = 'sat';
            }

            var start_date_time     = new Date(year, month, day, from[0], from[1]);
            var end_date_time       = new Date(year, month, day, to[0], to[1]);

            var start_timestamp     = start_date_time.getTime();
            var end_timestamp       = ( end_date_time.getTime() + 60000 );
            var current_timestamp   = current_date_time.getTime();

            if( current_timestamp < end_timestamp && start_timestamp < current_timestamp && days.indexOf(dayname) != -1 ) 
            {

                layoutName = 'schedule' + val.id + 'layout.html';

                if( val.screen == 0 )
                    screenOff = true;
                else
                    screenOff = false;
            }

        }
    }

    if( overrides ) 
    {
        for( var i = 0; i < overrides.length; i++ )
        {
            var val = overrides[i];


            var layout  = val.layout_id;
            var start_date = val.start_date.split('-');
            var from    = val.start_time;
                from    = from.split(':');
            var end_date = val.end_date.split('-');
            var to      = val.end_time;
                to      = to.split(':');

            var year    = current_date_time.getFullYear();
            var month   = current_date_time.getMonth();
            var day     = current_date_time.getDate();
            var dayname = current_date_time.getDay();
            var hour    = current_date_time.getHours();
            var minute  = current_date_time.getMinutes();
            var second  = current_date_time.getSeconds();

            if( start_date[1].substr(0,1) == '0' )
                start_date[1] = start_date[1].substr(1,2);

            if( start_date[2].substr(0,1) == '0' )
                start_date[2] = start_date[2].substr(1,2);

            if( end_date[1].substr(0,1) == '0' )
                end_date[1] = end_date[1].substr(1,2);

            if( end_date[2].substr(0,1) == '0' )
                end_date[2] = end_date[2].substr(1,2);

            var start_date_time     = new Date(start_date[0], ( parseFloat( start_date[1] ) - 1 ), start_date[2], from[0], from[1]);
            var end_date_time       = new Date(end_date[0], ( parseFloat( end_date[1] ) - 1 ), end_date[2], to[0], to[1]);

            var start_timestamp     = start_date_time.getTime();
            var end_timestamp       = ( end_date_time.getTime() + 60000 );
            var current_timestamp   = current_date_time.getTime();

            if( current_timestamp < end_timestamp && start_timestamp < current_timestamp ) {

                layoutName = 'override' + val.id + 'layout.html';

                if( val.screen == 0 )
                    screenOff = true;
                else
                    screenOff = false;

            }

        }
    }

    if( layoutName == '' )
        layoutName = 'embedapp.html';

    // if( file_name != window.slug + '.html' )
    // {
        app.layout.show( layoutName, force );
    // }

};