var EmbedConnect = function() {

    this.init();

    return this;

};

EmbedConnect.prototype.request = false;

EmbedConnect.prototype.init = function()
{

    // this.load();

};

EmbedConnect.prototype.setTimeout = function()
{

    if( this.timeout )
    {
        clearTimeout( this.timeout );
    }

    this.timeout = setTimeout( this.load.bind(this), parseInt(app.device.CMSData.update_time) );

};

EmbedConnect.prototype.load = function()
{

    if( !this.request )
    {   
        this.request = new XMLHttpRequest();
    }

    this.request.onerror    = this.error.bind(this);

    this.request.ontimeout    = this.error.bind(this);

    this.request.onload     = this.success.bind(this);

    app.device.details.stats.time = new Date().toString();

    if( app.brightSign.storage )
    {
        app.device.details.stats.storageSpace = app.file.formatBytes(app.brightSign.storage.free, 1);
        app.device.details.stats.totalStorageSpace = app.file.formatBytes(app.brightSign.storage.total, 1);
    }


    // need to do same as other, var 

    var availableInterface = false;


    for( var i = 0; i < app.brightSign.availableInterfaces.length; i++ )
    {

        if( app.brightSign.currentNetworkConfig.networkinterface == app.brightSign.availableInterfaces[i].interface )
        {
            availableInterface = app.brightSign.availableInterfaces[i];
        }

    }

    if( !availableInterface && app.brightSign.availableInterfaces.length )
    {
        availableInterface = app.brightSign.availableInterfaces[0];
    }

    if( availableInterface )
    {
        app.device.details.stats.macAddress = availableInterface.mac;
        app.device.details.stats.ipAddress    = availableInterface.address;
        app.device.details.stats.subnetMask = availableInterface.netmask;


        if( availableInterface.dns && availableInterface.dns.length )
        {

            var dnsValue = '';

            for( var e = 0; e < availableInterface.dns.length; e++ )
            {

                if( e == 0 )
                {
                    dnsValue = dnsValue += availableInterface.dns[e];
                }
                else
                {
                    dnsValue = dnsValue += ',' + availableInterface.dns[e];  
                }

            }

            app.device.details.stats.dns = dnsValue;

        }

        app.device.details.connection = availableInterface.interface;
    }

    if( app.brightSign.ram )
    {

        var percent     = ( ( app.brightSign.ram.free / app.brightSign.ram.total ) * 100 ).toFixed(2);
        var available   = app.file.formatBytes( app.brightSign.ram.free, 2 );
        var total       = app.file.formatBytes( app.brightSign.ram.total, 2 );

        app.device.details.stats.ram = available + ' / ' + total + ' ' + '(' + percent + '%)';

    }

    if( app.brightSign.cpu )
    {

        app.device.details.stats.cpu = app.brightSign.cpu.percent + '% (' + app.brightSign.cpu.count + ' CPUs)';

    }


    // app.device.details.stats.availableStorage = app.file.details.availableStorage;

    var params = this.getPostData(
        {
            'device' : app.device.details,
            'requests' : []
        }
    );

    this.post( app.config.url + '/api/v2/' + app.device.accountKey + '/connect', params );

};

EmbedConnect.prototype.success = function()
{

    if( this.request.status >= 200 && this.request.status < 400 ) 
    {
        
        var data = JSON.parse( this.request.responseText );

        this.online( data );

    } 
    else 
    {

        console.log('Embed: Request success error');

        this.offline();

    }
    
};

EmbedConnect.prototype.error = function()
{
    console.log('Embed: Request error');

    this.offline();

};

EmbedConnect.prototype.online = function( data )
{

    app.channel.cancelDownload = false;

    this.processResponse( data );

    if( data.response.usesWebsockets )
    {
        app.controller.enable();
    }

    this.setTimeout();

};

EmbedConnect.prototype.offline = function()
{

    app.channel.cancelDownload = true;

    app.file.read( 'info', 'fileList.txt', this.checkContent, true );

    // console.log('offline');

    app.controller.disable();

    this.checkLicense();

    this.setTimeout();

};

EmbedConnect.prototype.checkContent = function( success, content, sPath )
{

    if( success && content && content != '' && content != '[]' )
    {


        // console.log('content files exist');

    }
    else
    {

        app.messages.show('noInternetConnection');

    }
    

};

EmbedConnect.prototype.checkLicense = function()
{

    if( app.device.CMSData && app.device.CMSData.device && app.device.CMSData.device.last_connected )
    {

        var currentDate     = new Date().getTime();

        // New changes to 30 day,
        // If license exists, check current timestamp vs valid_until, if later than valid_until + 2 weeks, shut off.
        // Otherwise if no license exists, do same as before off the last_connected.
        // if( app.device.CMSData.device.license && app.device.CMSData.device.license.key && app.device.CMSData.device.license.valid_until )
        // {
        //     var licenseExpiry   = new Date( app.device.license.valid_until ).getTime();

        //     if( currentDate > licenseExpiry )
        //     {
                
        //         var difference      = currentDate - licenseExpiry;
        //         var days            = Math.round(difference / (1000 * 60 * 60 * 24)-1);

        //         // Two week grace
        //         if( !app.device.CMSData.status || days > 14 )
        //         {
        //             app.messages.show( 'expired' );
        //             app.channel.clear();
        //         }

        //     }
        // }
        // else
        // {
            var lastConnected   = new Date( app.device.CMSData.device.last_connected ).getTime();

            var difference      = currentDate - lastConnected;
            var days            = Math.round(difference / (1000 * 60 * 60 * 24)-1);

            // Change this to license end date.
            if( !app.device.CMSData.status || days > 28 )
            {

                app.messages.show( 'expired' );
                app.channel.clear();

            }
        // }
        

    }
    else
    {

        app.messages.show('noInternetConnection');

    }

};

EmbedConnect.prototype.post = function( url, params )
{

    this.request.open( 'POST', url, true );

    this.request.setRequestHeader("Content-type", "application/x-www-form-urlencoded");

    this.request.send(params);

};

EmbedConnect.prototype.get = function( url, params )
{

    this.request.open( 'GET', url + '?' + params, true );

    this.request.onerror    = this.error.bind(this);
    this.request.onload     = this.success.bind(this);

    this.request.send();
    
};

EmbedConnect.prototype.buildParams = function (prefix, obj, traditional, add) {

    if( Array.isArray(obj) )
    {

        for( var objKey in obj )
        {

            if( traditional || /\[\]$/.test( prefix ) ) 
            {

                add( prefix, obj[ objKey ] );

            } 
            else 
            {

                this.buildParams( prefix + "[" + ( typeof obj[ objKey ] === "object" || Array.isArray( obj[ objKey ] ) ? objKey : "" ) + "]", obj[ objKey ], traditional, add );

            }

        }

    } 
    else if( !traditional && obj != null && typeof obj === "object" ) 
    {

        for( var objKey in obj ) 
        {

            this.buildParams( prefix + "[" + objKey + "]", obj[ objKey ], traditional, add );

        }

    } 
    else 
    {

        add( prefix, obj );

    }

};

EmbedConnect.prototype.getPostData = function (a) {

    var s = [], add = function (key, value) {

        s[s.length] = encodeURIComponent(key) + "=" + encodeURIComponent(value);

    };

    for (var prefix in a) {

        this.buildParams(prefix, a[prefix], false, add);

    }

    // Return the resulting serialization
    return s.join("&").replace(/%20/g, "+");

};

EmbedConnect.prototype.processResponse = function( data )
{

    var check = data.response.check;
    var files = data.response.files;

    app.device.saveCMSData( data );

    if( app.device.CMSData.registered )
    {
        app.loadAnalytics(app.device.details.embedID, data.response.device.license.key);
    }

    if( !app.device.CMSData.registered )
    {
        app.channel.clear();

        app.messages.messages.code.title = app.device.CMSData.unique;
        app.messages.show( 'code' );
    }
    else if( app.device.CMSData.status == 0 )
    {
        this.checkLicense();
    }
    else if( !app.device.CMSData.channel.schedules )
    {
        app.channel.clear();

        app.messages.show( 'registered' );
    }
    else if( !app.device.CMSData.files )
    {
        app.channel.clear();
        app.messages.show( 'noFiles' );
    }
    else if( files )
    {
        
        var updates = app.channel.checkMD5( check );

        if( updates )
        {

            app.messages.show( 'downloading' );

            app.channel.update( files, check );

        }

        // console.log('has channel');

    }
    // else if( this.response.channel.length )
    // {
    //  app.messages.show( 'registered' );
    // }


}