/*

 Example on how to use
 ----------------------

 var MyController = new EmbedController("ws://websockets.embedsignage.com:8080", myDevice.deviceID, myApp.accountKey);


 //Enable / disable websockets
 if(mySettings.websocketsEnabled){

 MyController.enable();

 }else{

 MyController.disable();

 }



 MyController.onVolumeUp = function(){

 myDevice.volume += 1;

 };

 MyController.onVolumeDown = function(){

 myDevice.volume -= 1;

 };

 MyController.onScreenOff = function(){

 myDevice.turnScreenOff();

 }

 MyController.onGetVolume = function(){

 var deviceVolume = myDevice.getVolume();

 return deviceVolume;

 }

 //this shows how it is done on samsung screens.
 MyController.onGetScreenshot = function(){

 var screenshotURL = myDevice.getScreenshot();

 var base64EncodedImage = myFunctions.base64Encode(screenshotURL);

 myFunctions.ajax("POST", "https://app.embedsignage.com/api/v1/uploadDevicePreview", {
 "base64data": base64EncodedImage,
 "deviceid": myDevice.deviceID
 }, function () {
 //success
 }, function (e) {
 //error
 }, function () {
 //complete
 });

 }


 */

 /**
    *   embed signage controller function. Allows connection to the web controller via websockets in the CMS.
    *   @constructor
    * @param {string} webSocketURL - Url for the websocket connection.
  * @param {string} deviceID - The ID of the device.
  * @param {string} accountKey - The account key of the account the device is associated with. For API access.
 */
var EmbedController = function (webSocketURL, deviceID, accountKey) {

    // console.log("Controller created");

    // console.log(deviceID);

    // console.log(accountKey);

    // console.log(webSocketURL);

    this.settings = {

        "socketAddress": webSocketURL,
        "socketConnection": false,
        "appName": "SamsungControl",
        "timeout": null,
        "quitState": false,
        "deviceID": deviceID,
        "accountKey":accountKey || false,
        "features":[],
        "init":false,
        "open":false,
        "timeoutInit":null

    };


    // NOT SUPPORTED YET WILL BE IN CONTROLLER 2.0
    // //onGetVolume - Needs to return the volume in integer form
    // this.onGetVolume = false;
    //
    // //onGetScreenDisplayState - Returns true (screen on) or false (screen off)
    // this.onGetScreenDisplayState = false;

    //onGetDeviceInfo - returns object via callback {"volume":0,"screenOn":true}
    this.onGetDeviceInfo = false;

    //onChannelUp / onChannelDown - returns nothing is called when someone clicks channel up/down in device controller do with this what you would like
    this.onChannelUp = false;
    this.onChannelDown = false;

    //onVolumeUp / onVolumeDown - returns nothing is called when someone clicks volume up/down in device controller do with this what you would like
    this.onVolumeUp = false;
    this.onVolumeDown = false;

    //onSetVolume - passes one variable to function which is an integer between 0 and 100
    this.onSetVolume = false;

    //onMuteOn / onMuteOff - returns nothing do with this as you would like
    this.onMuteOn = false;
    this.onMuteOff = false;

    //onScreenOn / onScreenOff - returns nothing do with this as you would like
    this.onScreenOn = false;
    this.onScreenOff = false;

    //onPowerOff - returns nothing do with this as you would like
    this.onPowerOff = false;

    //onReboot - returns nothing do with this as you would like
    this.onReboot = false;

    this.onSetTime = false;
    this.onSetTimer = false;

    //onGetScreenshot - returns nothing do with this as you would like
    this.onGetScreenshot = false;

    this.onGetLogs = false;

    this.init();

};

EmbedController.prototype.setAPIKey = function(apiKey, deviceID){

    if(!this.settings.init){

        this.settings.deviceID = deviceID;

        this.settings.accountKey = apiKey;

        this.init();

    }else{

        if(this.settings.accountKey !== apiKey || this.settings.deviceID !== deviceID){

            this.settings.deviceID = deviceID;

            this.settings.accountKey = apiKey;

            this.close();

        }

    }

};

EmbedController.prototype.enable = function () {

    this.settings.enabled = true;

};

EmbedController.prototype.disable = function () {

    this.settings.enabled = false;

    this.close();

};

EmbedController.prototype.close = function () {

    // console.log("Closing socket connection");

    try{

        var cArray = [2,3];

        if(this.settings.socketConnection && cArray.indexOf(this.settings.socketConnection.readyState) === -1){
            this.settings.socketConnection.close();
        }

        // console.log(this.settings.socketConnection);

    }catch(e){

    }

};

/*

 This allows us to send a message with an ID this means we can communicate.
 Between devices but is only used to return response to embed curerntly.

 */
EmbedController.prototype.sendMessage = function(id, action, actionData){

    try{

        // console.log("Send Message");
        // console.log(id);
        // console.log(action);

        if(!actionData){
            actionData = "";
        }

        if(this.settings.open){

            var newAction = {
                "action":action,
                "actionData":actionData,
                "app":"SamsungControl",
                "device_id":this.settings.deviceID,
                "devices":[id]
            };

            var cArray = [0,1];

            if(cArray.indexOf(this.settings.socketConnection.readyState) !== -1 ){

                this.settings.socketConnection.send( JSON.stringify(newAction) );

            }

        }

    }catch(e){

    }

};



EmbedController.prototype.getAvailableFeatures = function(){

    if(typeof this.onChannelUp === "function" && typeof this.onChannelDown === "function"){
        this.settings.features.push("channelUpDown");
    }

    if(typeof this.onSetVolume === "function"){
        this.settings.features.push("volume");
    }

    if(typeof this.onMuteOff === "function" && typeof this.onMuteOn === "function"){
        this.settings.features.push("muteOnOff");
    }

    if(typeof this.onScreenOff === "function" && typeof this.onScreenOn === "function"){
        this.settings.features.push("screenOnOff");
    }

    if(typeof this.onPowerOff === "function"){
        this.settings.features.push("powerOnOff");
    }

    if(typeof this.onReboot === "function"){
        this.settings.features.push("reboot");
    }

    if(typeof this.onGetScreenshot === "function"){
        this.settings.features.push("screenshot");
    }

    if(typeof this.onSetTime === "function"){
        this.settings.features.push("setTime");
    }

    if(typeof this.onSetTimer === "function"){
        this.settings.features.push("onOffTimer");
    }

    if(typeof this.onGetLogs === "function"){
        this.settings.features.push("logs");
    }

};

EmbedController.prototype.init = function(){

    // console.log("Connection initial-scaleated!");
    if(this.settings.accountKey){

        // console.log("Account key set!");

        this.settings.init = true;

        this.initConnection();

        setTimeout(function(){

            this.getAvailableFeatures();

        }.bind(this), 2000);

    }

};

/*

 start the connection if not started is called every 8 seconds to check that we are still connected

 */
EmbedController.prototype.initConnection = function () {

    // console.log("Init connection");

    var self = this;

    try {

        if( this.settings.enabled ) 
        {

            var cArray = [1, 0]; // connecting or connected status codes

            // this.settings.quitState = (!this.settings.socketConnection || cArray.indexOf(this.settings.socketConnection.readyState) === -1) ? false : true;

            if( !this.settings.socketConnection || cArray.indexOf(this.settings.socketConnection.readyState) === -1 )
            {

                // if (!this.settings.quitState) {

                // console.log("Create socket address!");

                this.settings.socketConnection = new WebSocket(this.settings.socketAddress);

                this.settings.socketConnection.onopen = this.websocketOpen.bind(this);
                this.settings.socketConnection.onmessage = this.websocketMessage.bind(this);
                this.settings.socketConnection.onerror = this.websocketError.bind(this);
                this.settings.socketConnection.onclose = this.websocketClose.bind(this);

            }

        } else {

            /*
             ** close connection if connection exists as websockets has now been disabled.
             */
            // if (this.settings.quitState) {

            //     this.settings.socketConnection.close();

            // }

            clearTimeout(this.settings.timeout);
            this.settings.timeout = setTimeout(this.initConnection.bind(this), 60000);

        }

        // this.timeout = setTimeout(function () {

        //     self.timeout = 0;

        //     self.initConnection();

        // }, 8000);

    } catch (e) {

        console.log("wss error! : " + e);

        clearTimeout(this.settings.timeout);
        this.settings.timeout = setTimeout(this.initConnection.bind(this), 10000);

    }

};

EmbedController.prototype.websocketOpen = function (e) {

    this.settings.open = true;

    var registerBrowserJSON = {
        "action": "store_device_id",
        "accountKey":this.settings.accountKey,
        "device_id": this.settings.deviceID,
        "app": "SamsungControl"
    };

    // console.log("OPEN CONNECTION");
    // console.log(e);
    //
    // console.log(registerBrowserJSON);

    this.settings.socketConnection.send(JSON.stringify(registerBrowserJSON));

};

EmbedController.prototype.websocketMessage = function (e) {

    this.processMessage(JSON.parse(e.data));

};

EmbedController.prototype.websocketError = function (e) {

    console.log("Socket error!");
    // console.log(e);

};

EmbedController.prototype.websocketClose = function (e) {

    console.log("On close");
    // // console.log(this.settings.socketConnection);
    // console.log(e);


    this.settings.open = false;

    this.settings.socketConnection = null;

    clearTimeout(this.settings.timeout);
    this.settings.timeout = setTimeout(this.initConnection.bind(this), 10000);

};

EmbedController.prototype.processMessage = function (message) {

    if (message.action) {

        switch (message.action) {

            case "setvolume":

                if(typeof this.onSetVolume === "function"){

                    if(message.actionData){
                        this.onSetVolume(message.actionData);
                    }

                }

                break;

            case "channelup":

                if (typeof this.onChannelUp === "function") {

                    this.onChannelUp();

                }

                break;

            case "channeldown":

                if (typeof this.onChannelDown === "function") {

                    this.onChannelDown();

                }

                break;

            case "volumeup":

                if (typeof this.onVolumeUp === "function") {

                    this.onVolumeUp();

                }

                break;

            case "volumedown":

                if (typeof this.onVolumeDown === "function") {

                    this.onVolumeDown();

                }

                break;

            case "volume":

                if (typeof this.onVolumeUp === "function") {

                    this.onVolumeUp();

                }

                break;

            case "muteon":

                if (typeof this.onMuteOn === "function") {

                    this.onMuteOn();

                }

                break;

            case "muteoff":

                if (typeof this.onMuteOff === "function") {

                    this.onMuteOff();

                }

                break;

            case "screenon":

                if (typeof this.onScreenOn === "function") {

                    this.onScreenOn();

                }

                break;

            case "screenoff":

                if (typeof this.onScreenOff === "function") {

                    this.onScreenOff();

                }

                break;

            case "poweroff":

                if (typeof this.onPowerOff === "function") {

                    this.onPowerOff();

                }

                break;

            case "reboot":

                if (typeof this.onReboot === "function") {

                    this.onReboot();

                }

                break;

            case "setTime":

                    if(typeof this.onSetTime === "function"){

                        if(message.actionData){
                            this.onSetTime(message.actionData);
                        }

                    }

                break;

            case "setTimer":

                    if(typeof this.onSetTimer === "function"){

                        if(message.actionData){
                            this.onSetTimer(message.actionData);
                        }

                    }

                break;

            case "getdeviceinfo" :

                if(typeof this.onGetDeviceInfo === "function"){

                    //Please return an object with the volume of the application and the screenState
                    //Features are a list of available functions that device controller can use with the device.
                    //{
                    // "screenOn":false,
                    // "volume" : 58
                    //}

                    this.onGetDeviceInfo(function(information){

                        //insert available features into the features list.

                        // "features":["screenshot","reboot","screenOnOff","powerOnOff","reboot","muteOnOff","volume","channelUpDown"]
                        information.features = this.settings.features;

                        this.sendMessage(message.device_id, "deviceInfo", information);

                    }.bind(this));

                }

                break;

            case "getpreview":

                if (typeof this.onGetScreenshot === "function") {

                    this.onGetScreenshot(function(){

                        this.sendMessage(message.device_id, "screenshotUpdated");

                    }.bind(this));

                }

                break;

            case "getlogs":

                if (typeof this.onGetLogs === "function") {

                    this.onGetLogs(function(){

                        this.sendMessage(message.device_id, "logsUpdated");

                    }.bind(this));

                }

                break;

        }

    }

};