var EmbedDevice = function() {

    this.init();

    return this;

};

EmbedDevice.prototype.accountkey = 'DB5C1A124ABC92876B252E4B4A586';

EmbedDevice.prototype.details = {
    'name'          : '',
    'embedID'       : '',
    'appVersion'    : '1.0.0',
    'platform'      : 'BrightSign',
    'model'         : '',
    'software'      : '',
    'connection'    : 'wifi',
    'location'      : {
        'geoLocal'  : false,
        'lat'       : 0,
        'long'      : 0,
    },
    'width'         : 0,
    'height'        : 0,
    'stats'         : {
        'cpu'               : 'N/A',
        'ram'               : 'N/A',
        'temp'              : 'N/A',
        'channelReceived'   : '',
        'storageSpace'      : '',
        'ipAddress'         : 'N/A',
        'macAddress'        : 'N/A',
        'dns'               : 'N/A',
        'ipMode'            : 'N/A',
        'subnetMask'        : 'N/A',
        'gateway'           : 'N/A',
        'dnsMode'           : 'N/A',
        'bootTime'          : '',
        'time'              : ''
    }
};

EmbedDevice.prototype.init = function()
{

    this.getScreenInfo();

    window.addEventListener('resize', this.getScreenInfo.bind(this));

    this.getDeviceInfo();
    this.loadCMSData();

};

EmbedDevice.prototype.getScreenInfo = function()
{

    // this.details.height = document.body.style.height.replace('px', '');
    // this.details.width  = document.body.style.width.replace('px', '');

    // if( app.brightSign.orientation == 'portrait' || app.brightSign.orientation == 'reversedPortrait' )
    // {
    //     this.details.height = document.body.style.width.replace('px', '');
    //     this.details.width  = document.body.style.height.replace('px', '');
    // }

    this.details.height = document.body.clientHeight;
    this.details.width  = document.body.clientWidth;

    if( app.brightSign.orientation == 'portrait' || app.brightSign.orientation == 'reversedPortrait' )
    {
        this.details.height = document.body.clientWidth;
        this.details.width  = document.body.clientHeight;
    }

};

EmbedDevice.prototype.getDeviceInfo = function()
{

	var device = new BSDeviceInfo();

    this.details.name       = 'BrightSign ' + device.model;
    this.details.embedID    = 'BRIGHT' + device.deviceUniqueId.replace(/\-/g, '');

    this.details.model      = device.model;

    this.details.appVersion = app.config.version;
    this.details.software   = device.version + '/' + device.bootVersion + '/' + device.family;

    // this.details.connection = navigator.connection.type;

    this.details.serial     = device.deviceUniqueId;
    this.details.upTime 	= device.deviceUptime;
    this.details.lifeTime 	= device.deviceLifetime;
    this.details.bootCount 	= device.deviceBootCount;

    this.details.family     = device.family;

    // chrome.system.cpu.getInfo( this.updateCPUInfo.bind(this) );

    // chrome.system.memory.getInfo( this.updateRamInfo.bind(this) );

    // chrome.system.network.getNetworkInterfaces( this.updateIPAddress.bind(this) );

    // if( app.file.ready )
    // {
    //     app.file.getAvailableStorage();
    // }

    window.embedDevice = {
        'uuid'      : this.details.embedID,
        'type'      : 'BrightSign',
        'platform'  : this.details.platform,
        'app_version' : this.details.appVersion,
        'version'   : this.details.software,
        'connection': this.details.connection,
        'storageUrl': ''
    };

    this.getGeolocation();

    clearTimeout(this.deviceInfoTimeout);
    this.deviceInfoTimeout = setTimeout( this.getDeviceInfo.bind(this), 60000 );

};

EmbedDevice.prototype.updateIPAddress = function( nInterfaces )
{

    var macFound = false;

    for( var i = 0; i < nInterfaces.length; i++ )
    {

        if( ['eth0', 'en0', 'wlan0'].indexOf( nInterfaces[i].name ) != -1 )
        {

            if( nInterfaces[i].prefixLength == 64 && !macFound )
            {
                macFound = true;
                this.details.stats.macAddress = nInterfaces[i].address;
            }

            if( nInterfaces[i].prefixLength == 24 )
            {
                this.details.stats.ipAddress = nInterfaces[i].address;
            }

        }

    }

};

EmbedDevice.prototype.updateMACAddress = function( message )
{
    this.details.stats.macAddress = message;
};

EmbedDevice.prototype.networkInfoFailed = function()
{

};

EmbedDevice.prototype.updateCPUInfo = function( info )
{
    if( info.processors )
    {

        var totalIdle = 0;
        var total = 0;

        for( var i = 0; i < info.processors.length; i++ )
        {

            totalIdle   = totalIdle + info.processors[i].usage.idle;
            total       = total + info.processors[i].usage.total;

        }

        var used = total - totalIdle;

        var percent     = ( ( used / total ) * 100 ).toFixed(2);

        // var available    = app.file.formatBytes( totalIdle, 2 );
        // var total        = app.file.formatBytes( info.capacity, 2 );

        this.details.stats.cpu = percent + '% (' + info.numOfProcessors + ' CPUs)';

    }
};

EmbedDevice.prototype.updateRamInfo = function( info )
{

    if( info.availableCapacity && info.capacity )
    {

        var percent     = ( ( info.availableCapacity / info.capacity ) * 100 ).toFixed(2);
        var available   = app.file.formatBytes( info.availableCapacity, 2 );
        var total       = app.file.formatBytes( info.capacity, 2 );

        this.details.stats.ram = available + ' / ' + total + ' ' + '(' + percent + '%)';

    }
};

EmbedDevice.prototype.getGeolocation = function()
{

    this.geolocationError();
	// if( navigator.geolocation )
	// {
    //     navigator.geolocation.getCurrentPosition( 
    //         this.geolocationSuccess.bind(this), 
    //         this.geolocationError.bind(this), 
    //         {
    //             timeout: 5000, 
    //             enableHighAccuracy: true
    //         }
    //     );
    // }
    // else
    // {
    // 	this.geolocationError();
    // }

};

EmbedDevice.prototype.geolocationSuccess = function( info )
{

    this.geolocationFinished = true;

    this.details.location.lat   = info.coords.latitude;
    this.details.location.long  = info.coords.longitude;

    this.details.location.geoLocal = true;

};

EmbedDevice.prototype.geolocationError = function( info )
{

    this.geolocationFinished = true;

    this.details.location.lat   = 0;
    this.details.location.long  = 0;

    this.details.location.geoLocal = false;

};

EmbedDevice.prototype.loadCMSData = function()
{

    app.file.read( 'info', 'cmsData.txt', this.CMSDataReadResponse.bind(this), true );

    // app.brightSign.classes.registry.read('embedInfo', 'cmsData').then(function( info ) {

    //     if( !info || info == '' )
    //     {

    //         this.CMSDataReadResponse( false, '' );

    //     }
    //     else
    //     {

    //         this.CMSDataReadResponse( true, info );
            
    //     }

    // }.bind(this)).catch( function(e) {
    //     this.CMSDataReadResponse( false, '' );
    // } );

    
};

EmbedDevice.prototype.CMSDataReadResponse = function( success, contents, lurl )
{

	// {"registered":true,"channel":[],"files":[],"usesWebsockets":1,"update_time":"30000","analytics_time":0,"unique":"","device":{"id":10842,"name":"BrightSign XT1143","settings":{"volume":"0","storage":"internal","orientation":"portrait","timeSync":"2018-06-14 17:42:33"},"actions":"","last_connected":{"date":"2018-06-14 16:42:33.000000","timezone_type":3,"timezone":"UTC"},"account":{"company_name":"EDM","account_key":"547d7908e56cc","analytics_enabled":0}},"status":1}
    if( success && contents && contents != '' )
    {

        if( typeof contents == 'string' )
        {
            try {
                this.CMSData = JSON.parse(contents);
            }
            catch(e)
            {
                this.CMSData = {
                    'device' : {
                        'account' : {
                            'account_key' : 'DB5C1A124ABC92876B252E4B4A586',
                        }
                    },
                    'update_time' : 30000,
                    'settings' : {
                        'volume' : 100,
                        'storage' : 'internal'
                    }
                };
            }
        }
        else
        {
            this.CMSData = contents;

            // getting here
        }

        if( this.CMSData.response )
        {
            this.CMSData = this.CMSData.response;
        }

    }
    else
    {   
        this.CMSData = {
            'device' : {
                'account' : {
                    'account_key' : 'DB5C1A124ABC92876B252E4B4A586',
                }
            },
            'update_time' : 30000,
            'settings' : {
                'volume' : 100,
                'storage' : 'internal'
            }
        };
    }

    

    if( this.CMSData.device.account && this.CMSData.device.account.account_key )
    {

        this.accountKey = this.CMSData.device.account.account_key;

    }
    else
    {
        this.accountKey = 'DB5C1A124ABC92876B252E4B4A586';
    }

    if( this.CMSData.device && this.CMSData.device.stats )
    {

        this.details.stats = this.CMSData.device.stats;

    }

    // console.log(this.CMSData);

};

EmbedDevice.prototype.saveCMSData = function( response )
{

    if( !this.savingData )
    {

        this.savingData = true;

        var oldData = this.CMSData;
        
        if( response && !app.troubleshooting )
        {

            var check = this.CMSData.check;
            // pretty sure this is where stuff gets wiped
            this.CMSData = response.response;
            this.CMSData.check = check;

            if( this.CMSData.device && oldData.device )
            {
                this.CMSData.device.stats = app.device.details.stats;
            }

        }

        if( this.CMSData.device.account && this.CMSData.device.account.account_key )
        {
            this.accountKey = this.CMSData.device.account.account_key;
        }
        else
        {
            this.accountKey = 'DB5C1A124ABC92876B252E4B4A586';
        }

        if( this.CMSData.device && this.CMSData.device.settings && typeof this.CMSData.device.settings == 'string' )
        {

            this.CMSData.device.settings = JSON.parse(this.CMSData.device.settings);

        }

        if( this.CMSData.device && this.CMSData.device.settings )
        {

            if( oldData && oldData.device && oldData.device.settings )
            {

                if( typeof oldData.device.settings == 'string' )
                {
                    oldData.device.settings = JSON.parse(oldData.device.settings);
                }

                app.brightSign.setScreenOrientation( this.CMSData.device.settings.orientation );

                if( this.CMSData.device.settings.videoMode )
                {
                    app.brightSign.overwriteResolution( this.CMSData.device.settings.videoMode );
                }

                if( this.CMSData.device.settings.multiOutputs )
                {
                    app.brightSign.overwriteOutputs( this.CMSData.device.settings.multiOutputs, false, true );
                }
                else if( app.device.details.stats.multiOutputsCMS == 'cms' )
                {
                    app.device.details.stats.multiOutputsCMS = 'updated';
                    app.brightSign.registryData.embedinfo.outputsoverridecms = 'updated';
                    app.brightSign.classes.registry.write('embedinfo', 'outputsoverridecms', 'updated').then( app.brightSign.registryWritten.bind(app.brightSign) ).catch( app.brightSign.promiseError.bind(app.brightSign) );
                }

                app.setRebootSchedule( oldData.device.settings.rebootSchedule, this.CMSData.device.settings.rebootSchedule );

                if( this.CMSData && this.CMSData.device && this.CMSData.device.settings && this.CMSData.device.settings.cecSchedule )
                {
                    app.setCecSchedule('', this.CMSData.device.settings.cecSchedule);
                }

                if( this.CMSData.device.settings.videoPlayer )
                {
                    app.updateVideoPlayer( this.CMSData.device.settings.videoPlayer );
                }
            }
            else
            {
                app.brightSign.setScreenOrientation( this.CMSData.device.settings.orientation );

                if( this.CMSData.device.settings.videoMode )
                {
                    app.brightSign.overwriteResolution( this.CMSData.device.settings.videoMode );
                }
                
                if( this.CMSData.device.settings.multiOutputs && JSON.stringify(this.CMSData.device.settings.multiOutputs) != app.brightSign.registryData.embedinfo.outputsoverride )
                {
                    app.brightSign.overwriteOutputs( this.CMSData.device.settings.multiOutputs, false, true );
                }
                else if( app.device.details.stats.multiOutputsCMS == 'cms' )
                {
                    app.device.details.stats.multiOutputsCMS = 'updated';
                    app.brightSign.registryData.embedinfo.outputsoverridecms = 'updated';
                    app.brightSign.classes.registry.write('embedinfo', 'outputsoverridecms', 'updated').then( app.brightSign.registryWritten.bind(app.brightSign) ).catch( app.brightSign.promiseError.bind(app.brightSign) );
                }

                

                app.setRebootSchedule( '', this.CMSData.device.settings.rebootSchedule );

                if( this.CMSData && this.CMSData.device && this.CMSData.device.settings && this.CMSData.device.settings.cecSchedule )
                {
                    app.setCecSchedule('', this.CMSData.device.settings.cecSchedule);
                }

                if( this.CMSData.device.settings.videoPlayer )
                {
                    app.updateVideoPlayer( this.CMSData.device.settings.videoPlayer );
                }
            }

        }

        if( typeof this.CMSData.gmt_offset != 'undefined' )
        {

            if( oldData && typeof oldData.gmt_offset != 'undefined' )
            {
                app.brightSign.setTimezone( this.CMSData.gmt_offset );
            }
            else
            {
                app.brightSign.setTimezone( this.CMSData.gmt_offset );
            }

        }

        if( this.CMSData.device && this.CMSData.device.settings && this.CMSData.device.settings.debug == 1 && !this.isDebugDevice )
        {

            this.isDebugDevice = true;

            window.console = new EmbedConsole();

        }

        if( this.CMSData.device && this.CMSData.device.actions && typeof this.CMSData.device.actions == 'string' )
        {
            this.CMSData.device.actions = JSON.parse(this.CMSData.device.actions);
        }

        app.file.writeFile( 'info', 'cmsData.txt', JSON.stringify( this.CMSData ), this.CMSDataSaveResponse.bind(this) );

        // app.brightSign.classes.registry.write('embedInfo', 'cmsData', null).then( this.CMSDataSaveResponse.bind(this) ).catch( this.CMSDataSaveError.bind(this) );

        if( this.CMSData.device && this.CMSData.device.actions && this.CMSData.device.actions.length )
        {
            app.processActions();
        }

        app.updateInfo();

    }

};

EmbedDevice.prototype.clearCMSData = function( callback )
{

    app.file.deleteFile('info', 'cmsData.txt', function(success) {
            
        if( success )
        {
            app.brightSign.detectStorage();
        }

        if( callback )
        {
            callback();
        }

    });

};

EmbedDevice.prototype.CMSDataSaveResponse = function( success, url )
{

    this.savingData = false;

    if( !success )
    {
        console.log('CMSdata save error');
    }

};

EmbedDevice.prototype.CMSDataSaveError = function( err )
{

    this.savingData = false;
    console.log('CMSdata save error: ' + JSON.stringify(err));

};



