// ----------------------
// ---- File Manager ----
// ----------------------

var EmbedFileManager = function() {

    this.init();

    return this;

};

EmbedFileManager.prototype.details = {
    'space' : {
        'total' : 0,
        'free'  : 0
    }
};

EmbedFileManager.prototype.init = function()
{

    this.loadFileSystem();
    
    this.loadDirectories();

};

EmbedFileManager.prototype.loadFileSystem = function() 
{

	this.system = require('fs');

};

EmbedFileManager.prototype.systemLoaded = function( dirEntry ) 
{

    
        
};

EmbedFileManager.prototype.systemLoadError = function() 
{
    // console.log('Loading system failed');
};

EmbedFileManager.prototype.loadDirectories = function() 
{

    this.directories = {};

    this.directories[ 'root' ] = '/storage/sd/';

    this.system.mkdir('/storage/sd/info', this.directoryLoaded.bind({'this' : this, 'name' : 'info'}), this.directoryLoadError.bind({'this' : this, 'name' : 'info'}));

    this.system.mkdir('/storage/sd/tmp', this.directoryLoaded.bind({'this' : this, 'name' : 'tmp'}), this.directoryLoadError.bind({'this' : this, 'name' : 'tmp'}));

    this.system.mkdir('/storage/sd/updates', this.directoryLoaded.bind({'this' : this, 'name' : 'updates'}), this.directoryLoadError.bind({'this' : this, 'name' : 'updates'}));
    
    this.system.mkdir('/storage/sd/files', this.directoryLoaded.bind({'this' : this, 'name' : 'files'}), this.directoryLoadError.bind({'this' : this, 'name' : 'files'}));

};

EmbedFileManager.prototype.directoryLoaded = function() 
{

    this.this.directories[ this.name ] = '/storage/sd/' + this.name;

    if( this.this.directories.files && this.this.directories.tmp && this.this.directories.info )
    {

        this.this.ready = true;

        app.fileSystemReady();

    }

};

EmbedFileManager.prototype.directoryLoadError = function( err ) 
{
    if( err.code == 'EEXIST' )
    {
        this.this.directories[ this.name ] = '/storage/sd/' + this.name;

        if( this.this.directories.files && this.this.directories.tmp && this.this.directories.info )
        {

            this.this.ready = true;

            app.fileSystemReady();

        }
    }
    else
    {
        console.log('Loading directory failed : ' + JSON.stringify(err));
    }

};

EmbedFileManager.prototype.formatBytes = function( bytes, decimals )
{
    if (bytes === 0) return '0 Byte';
    var k = 1000; // or 1024 for binary
    var dm = decimals + 1 || 3;
    var sizes = ['Bytes', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'];
    var i = Math.floor(Math.log(bytes) / Math.log(k));
    return parseFloat((bytes / Math.pow(k, i)).toFixed(dm)) + ' ' + sizes[i];
};

EmbedFileManager.prototype.read = function(dir, fileName, callback, create) 
{

    if( typeof create == 'undefined' )
    {
        create = false;
    }

    var directory = this.directories[dir];

    if( typeof directory == 'undefined' )
    {
        directory = '/storage/sd/' + dir;
    }

    this.system.readFile( directory + '/' + fileName, "utf8",
        function(err, data) 
        {
            if( err )
            {
                
                console.log(err);

                callback(false);

            }
            else
            {
                // console.log(data);
                callback(true, data, directory + '/' + fileName);
            }
            
        }.bind(this)
    );

};

EmbedFileManager.prototype.writeFile = function(dir, fileName, contents, callback) 
{

    var directory = this.directories[dir];

    if( typeof directory == 'undefined' )
    {
        directory = '/storage/sd/' + dir;
    }
    
    this.system.writeFile( directory + '/' + fileName, contents, 
        function(err) 
        {
            if( err )
            {
                
                console.log(err);

                callback(false);

            }
            else
            {
                // console.log('file: ' + fileName + ' saved to directory: ' + this.directories[dir]);
                callback(true, directory + '/' + fileName);
            }
            
        }.bind(this)
    );

};

EmbedFileManager.prototype.downloadFile = function(url, dir, slug, callback, progress, length, path) 
{

    if( !length )
    {
        length = 0;
    }

    this.downloadDetails = {
        'url' : url,
        'dir' : dir,
        'slug': slug,
        'callback' : callback,
        'progress' : progress,
        'stats'     : {
            'downloaded' : 0,
            'length'     : length,
        }
    };

    if( path )
    {
        var directories = path.split('/');

        if( !this.system.existsSync('/storage/sd/tmp/' + path) )
        {
            this.system.mkdirSync('/storage/sd/tmp/' + path, {recursive: true});
        }
        // for( var i = 0; i < directories.length; i++ )
        // {
            

        //     currentPath += directories[i]
        // }

        path = path + '/';
    }
    else
    {
        path = '';
    }

    this.downloadDetails.writer = this.system.createWriteStream(this.directories[dir] + '/' + path + slug);

    // Uses fetch instead of XMLHttpRequest to support saving fragments into file as they
    // arrive. XMLHttpRequest will cause device to run out of memory when used with
    // large video files.
    fetch( url ).then( this.fetchResult.bind(this) ).catch( this.fetchError.bind(this) );

};

EmbedFileManager.prototype.fetchResult = function(res) 
{
    this.downloadDetails.stats.downloaded   = 0;

    if( res.headers.get('Content-Length') )
    {
        this.downloadDetails.stats.length       = res.headers.get('Content-Length');
    }
    
    // Content length might not be known, that is normal.
    // if( contentLength )
    //   console.log("Content length is " + contentLength);
    // else
    //   console.log("Content length is not known");
    //   
    // console.log(this.downloadDetails);

    return this.pump(res.body.getReader());
};

EmbedFileManager.prototype.fetchError = function(e) 
{
    console.log(e);
    this.downloadDetails.callback(false, e);
};

EmbedFileManager.prototype.downloadProgress = function() 
{
    if( this.downloadDetails.stats.length )
    {

        var response = {
            'total' : this.downloadDetails.stats.length,
            'loaded': this.downloadDetails.stats.downloaded,
            'lengthComputable' : true,
        };
    }
    else
    {
        var response = {
            'total' : 0,
            'loaded': this.downloadDetails.stats.downloaded,
            'lengthComputable' : false,
        };
    }

    this.downloadDetails.progress( response );
};

EmbedFileManager.prototype.pump = function( reader ) 
{

    this.downloadDetails.reader = reader;

    return this.downloadDetails.reader.read().then( this.pumpResult.bind(this) );

};

EmbedFileManager.prototype.pumpResult = function( result ) 
{

    // console.log(result);
    if( result.done ) 
    {
        // console.log("Download complete! " + this.downloadDetails.slug);

        this.downloadDetails.writer.end();

        this.downloadDetails.callback(true, this.directories[ this.downloadDetails.dir ] + '/' + this.downloadDetails.slug);
        
        return;
    }

    const chunk = result.value; // --> This is the chunked data
    
    this.downloadDetails.writer.write(new Buffer(chunk));

    this.downloadDetails.stats.downloaded += chunk.byteLength;
    // Console log takes long time, comment this out to download in full speed.
    this.downloadProgress();
    
    return this.pump(this.downloadDetails.reader);

};

EmbedFileManager.prototype.uploadFile = function(localUrl, remoteUrl, opts, callback) 
{
    

//  fileURL: Filesystem URL representing the file on the device or a data URI. For backwards compatibility, this can also be the full path of the file on the device. (See Backwards Compatibility Notes below)

// server: URL of the server to receive the file, as encoded by encodeURI().

// successCallback: A callback that is passed a FileUploadResult object. (Function)

// errorCallback: A callback that executes if an error occurs retrieving the FileUploadResult. Invoked with a FileTransferError object. (Function)

// options: Optional parameters (Object). Valid keys:

// fileKey: The name of the form element. Defaults to file. (DOMString)
// fileName: The file name to use when saving the file on the server. Defaults to image.jpg. (DOMString)
// httpMethod: The HTTP method to use - either PUT or POST. Defaults to POST. (DOMString)
// mimeType: The mime type of the data to upload. Defaults to image/jpeg. (DOMString)
// params: A set of optional key/value pairs to pass in the HTTP request. (Object, key/value - DOMString)
// chunkedMode: Whether to upload the data in chunked streaming mode. Defaults to true. (Boolean)
// headers: A map of header name/header values. Use an array to specify more than one value. On iOS, FireOS, and Android, if a header named Content-Type is present, multipart form data will NOT be used. (Object)
// trustAllHosts: Optional parameter, defaults to false. If set to true, it accepts all security certificates. This is useful since Android rejects self-signed security certificates. Not recommended for production use. Supported on Android and iOS. (boolean)
    
    var options = new FileUploadOptions();
    options.fileKey  = opts.key;
    options.fileName = fileURL.substr(fileURL.lastIndexOf('/') + 1);
    options.mimeType = opts.mimeType || "text/plain";
     
    var params = {};
    params.value1 = "test";
    params.value2 = "param";
     
    options.params = opts.params || [];
     
    var ft = new FileTransfer();
    
    ft.upload(
        localUrl, 
        encodeURI(remoteUrl), 
        this.uploadSuccess.bind({ 'callback' : callback }), 
        this.uploadFailed.bind({ 'callback' : callback }), 
        options
    );


};

EmbedFileManager.prototype.uploadSuccess = function( response ) 
{
    this.callback( true, response );
};

EmbedFileManager.prototype.uploadFailed = function( error ) 
{
    this.callback( false, error );
};

EmbedFileManager.prototype.deleteFile = function( dir, fileName, callback, path ) 
{

    var directory = this.directories[dir];

    if( typeof directory == 'undefined' )
    {
        directory = '/storage/sd/' + dir;
    }

    if( path )
    {
        path = path + '/';
    }
    else
    {
        path = '';
    }
    
    this.system.unlink( directory + '/' + path + fileName, function(err) {

        if( err )
        {
            callback(false);
        }
        else
        {
            callback(true);
        }

    } );

};

EmbedFileManager.prototype.moveFile = function( dir, fileName, newDir, newName, callback, path ) 
{

    var directory = this.directories[dir];

    if( typeof directory == 'undefined' )
    {
        directory = '/storage/sd/' + dir;
    }

    var newDirectory = this.directories[newDir];

    if( typeof directory == 'undefined' )
    {
        newDirectory = '/storage/sd/' + dir;
    }

    if( path )
    {

        if( !this.system.existsSync(newDirectory + '/' + path) )
        {
            this.system.mkdirSync(newDirectory + '/' + path, {recursive: true});
        }

        path = path + '/';
    }
    else
    {
        path = '';
    }

    this.system.rename( directory + '/' + path + fileName, newDirectory + '/' + path + newName, function(err) {

        if( err )
        {

            console.log(err);
            callback(false);

        }
        else
        {

            callback(true);

        }

    } );

};

EmbedFileManager.prototype.copyFile = function( dir, fileName, newDir, newName, callback ) 
{
    
    console.log('copy file not written');
    // var obj = {
    //     'this' : this,
    //     'newDir' : newDir,
    //     'newFileName' : newName,
    //     'callback'  : callback
    // };

    // this.system[dir].getFile(
    //     fileName, 
    //     { 
    //         create: true, 
    //         exclusive: false 
    //     }, 
    //     this.copyFileContents.bind(obj),
    //     this.copyFileError.bind(obj)
    // );

};

EmbedFileManager.prototype.emptyDirectory = function( dir, callback )
{

    var directory = this.directories[dir];

    if( typeof directory == 'undefined' )
    {
        directory = '/storage/sd/' + dir;
    }

    // if( !this.directories[ dir ] )
    // {
    //     console.log(dir + ', not loaded');
    //     callback(true);


    //     return;
    // }

    this.system.readdir( directory, function(err, files) {


        if( err )
        {
            callback(true);
        }
        else
        {

            for( var i = 0; i < files.length; i++ )
            {

                var curPath = directory + "/" + files[i];

                var stats = this.system.statSync( curPath );

                if( stats.isFile() )
                {

                    this.system.unlinkSync(curPath);


                }
                else if( stats.isDirectory() )
                {

                    this.system.rmSync(curPath, { recursive: true, force: true });

                }

            }

            callback(true);

        }


    }.bind(this) );

};