var Settings = function()
{

	setTimeout( this.init.bind(this), 5000 );

};

Settings.prototype.init = function()
{

	if( app.brightSign.families.s5.indexOf(app.device.details.family) == -1 || ( ( app.brightSign.registryData && app.brightSign.registryData.html && app.brightSign.registryData.html.enable_web_inspector == '1' ) || ( !app.brightSign.registryData || !app.brightSign.registryData.embedinfo || !app.brightSign.registryData.embedinfo.setupcomplete ) ) )
	{
		this.loadServer();
	}
	// this.loadServer();

};

Settings.prototype.loadServer = function()
{

	// Load the http module to create an http server.
	var http = require('http');
	var express = require('express');
	var session = require("express-session");
	this.passport = require('passport');
	var LocalStrategy = require('passport-local').Strategy;
	var cookieParser = require('cookie-parser');

	this.bcrypt = require('bcrypt-nodejs');

	var path = require("path");
	this.app = express();

	this.views = require("express-handlebars");

	var bodyParser = require('body-parser');

	this.app.engine('handlebars', this.views({
		defaultLayout: false
	}));

	this.app.set('view engine', 'handlebars');

	this.app.use( 
		bodyParser.urlencoded({
			extended: true
		})
	);

	this.app.use(cookieParser());


	this.userDetails = {
		'id'	: 1
	};

	this.userDetails.username = 'admin';
    this.userDetails.password = this.bcrypt.hashSync( app.device.details.serial, null, null );

    if( app.brightSign.registryData.embedauth && app.brightSign.registryData.embedauth.username && app.brightSign.registryData.embedauth.password )
    {

        this.userDetails.username = app.brightSign.registryData.embedauth.username;
        this.userDetails.password = app.brightSign.registryData.embedauth.password;

    }

    // console.log(this.userDetails);

  	 // required for passport
    this.app.use(session({
        secret: 'embedbsplayer',
        resave: true,
        saveUninitialized: true,
    })); // session secret

    this.passport.serializeUser(function(user, done)
    {

    	done(null, user.id);

    });

    this.passport.deserializeUser(function(id, done)
    {

    	// Write a logic to find this particular user from the json data using userID

        // If not found return done({});

        // else return done(null, userObject);
        // 

        

        if( id == this.userDetails.id )
    	{

    		return done(null, this.userDetails);

    	}
    	else
    	{
    		return done({});
    	}

    }.bind(this));

    this.passport.use(
        'local-login',
        new LocalStrategy({
            // by default, local strategy uses username and password, we will override with email
            usernameField: 'username',
            passwordField: 'password',
            passReqToCallback: true, // allows us to pass back the entire request to the callback
        }, function(req, username, password, done) { // callback with email and password from our form

        	if( username == this.userDetails.username && this.bcrypt.compareSync(password, this.userDetails.password) )
        	{

        		return done(null, this.userDetails);

        	}
        	else
        	{

        		return done(null, false);

        	}	

        }.bind(this))
    );

    this.app.use(this.passport.initialize());
    this.app.use(this.passport.session());

	this.app.use( bodyParser.json() );

	this.app.use('/settings', express.static(require("path").join('./', 'settings')));

	this.configureRoutes();

	this.server = http.createServer(this.app);

	this.server.on('error', this.serverError.bind(this));

	this.server.listen(8000);

	window.addEventListener('beforeunload', this.closeServer.bind(this));

};

Settings.prototype.closeServer = function()
{
	
	this.server.close();

	return 'Server closed';

};

Settings.prototype.checkAuth = function(req, res, next) 
{

	if( req.isAuthenticated() ) return next();

	res.redirect('/login');

};

Settings.prototype.checkLoggedIn = function(req, res, next)
{

	if( req.isAuthenticated() )
	{
		res.redirect('/admin');
	}
	else
	{
		return next();
	}

};

Settings.prototype.configureRoutes = function()
{

	this.app.get('/login', this.checkLoggedIn, function(req, res) {

		res.render('login');

	});

	this.app.get('/logout', this.checkAuth, function(req, res) {

		req.logout();
        res.redirect('/login');

	});

	this.app.post('/login', this.passport.authenticate('local-login', {
        successRedirect: '/', // redirect to the secure profile section
        failureRedirect: '/login', // redirect back to the signup page if there is an error
        failureFlash: true, // allow flash messages
    }), function( req, res ) {
            
        // console.log('hello');

        if( req.body.remember ) 
        {
            req.session.cookie.maxAge = 1000 * 60 * 3;
        } 
        else 
        {
            req.session.cookie.expires = false;
        }

        res.redirect('/');

    });

	this.app.get('/', this.checkAuth, function (req, res) {

		var networkConfig = app.brightSign.classes.network;
		

		app.brightSign.detectNetwork();

		this.data = { 
			'network' : {
				'ethernet' : {
					// 'config' : networkConfig.ethernet.getConfig(),
					'default': networkConfig.ethernet.defaultConfig()
				},
				'wifi'  : {
					// 'config' : networkConfig.wifi.getConfig(),
					'default': networkConfig.wifi.defaultConfig()
				},
				'modem' : {
					// 'config' : networkConfig.modem.getConfig(),
					'default': networkConfig.modem.defaultConfig()
				},
				'available' : app.brightSign.availableInterfaces,
				'current'	: app.brightSign.currentNetworkConfig
			},
			'stats' : {
				'storage' : app.brightSign.storage,
				'memory'  : app.brightSign.ram,
				'cpu'     : app.brightSign.cpu,
				'orientation' : app.brightSign.orientation,
				'resolution'  : app.brightSign.preferredScreenMode,
				'multiOutputs'  : app.brightSign.multipleOutputs,
				'availableRes': app.brightSign.availableScreenModes,
				'loadAvg'     : app.brightSign.loadAvg
			},
			// 'audioOutput' : app.brightSign.audioOutput,
			'device'  : app.device.details,
			'reg' : app.brightSign.registryData.embedinfo,
			// 'decoderConfig' : app.brightSign.decoderConfig
		};

		if( app.brightSign.registryData && app.brightSign.registryData.embedinfo && app.brightSign.registryData.embedinfo.setupcomplete )
	    {
	       	res.render('main', { 'data' : JSON.stringify(this.data) });    
	    }
	    else
	    {

	    	this.data.setup = true;

	        res.render('setup', { 'data' : JSON.stringify(this.data) });
	    }

		

		// networkConfig.ethernet.getConfig().then( 
		// 	function(response) {

		// 		this.data.network.ethernet.config = response;

		// 		return networkConfig.wifi.getConfig();

		// 	}.bind(this)
		// ).then( 
		// 	function( response ) {

		// 		this.data.network.wifi.config = response;

		// 		return networkConfig.modem.getConfig();

		// 	}.bind(this)
		// ).then( 
		// 	function( response ) {

		// 		this.data.network.modem.config = response;

		// 		res.render('main', { 'data' : JSON.stringify(this.data) });

		// 	}.bind(this)
		// ).catch( 
		// 	function(e) {
		// 		app.brightSign.updateNetwork();

		// 		res.render('main', { 'data' : JSON.stringify(this.data) });
		// 	}.bind(this)
		// );

	}.bind(this));


	this.app.get('/troubleshoot', this.checkAuth, function (req, res) {

		app.troubleshoot('local');

		res.render('troubleshoot');

	});

	this.app.get('/troubleshootplayer', this.checkAuth, function (req, res) {

		app.reset();

		res.render('troubleshoot');

	});
	

	this.app.post('/outputs', this.checkAuth, function (req, res) {

		var resolution  = req.body.resolutionOverride;
		// var audioOutput = req.body.audioOutput;

		if( app.brightSign.multipleOutputs )
		{
			setTimeout( function() {
				app.brightSign.overwriteOutputs( req.body.multiOutputs );
			}, 3000 );

			res.render('applyingchanges');
		}
		else
		{

			if( !app.brightSign.preferredScreenMode || resolution != app.brightSign.preferredScreenMode.modeName )
			{

				// setTimeout( function() {
					app.brightSign.overwriteResolution( resolution );
					// app.brightSign.updateAudioOutput( audioOutput );
				// }, 3000 );

				res.render('applyingchanges');

			}
			else
			{
				app.brightSign.overwriteResolution( resolution );
				// app.brightSign.updateAudioOutput( audioOutput );

				res.redirect('/');
			}
			
		}

	});


	this.app.post('/authentication', this.checkAuth, function (req, res) {

		var details = {
			'new' : {
				'username' : 'admin',
				'password' : req.body.newPassword
			},
			'old' : {
				'username' : 'admin',
				'password' : req.body.currentPassword
			}
		};

		var reboot = app.brightSign.updateCredentials( details );

		if( reboot )
		{
			res.render('applyingchanges');
		}
		else
		{
			res.redirect('/');
		}

	});

	//http://192.168.1.232:8080/runEthernetSetup?defaultGateway=192.168.1.1&ipAddress=192.168.1.232&networkTypeSelected=true&passphrase=&proxyAddress=&proxyPort=&subnetMask=255.255.255.0&timeServer=http:%2F%2Ftime.brightsignnetwork.com&useDHCP=true&useProxy=false

	this.app.post('/network', this.checkAuth, function (req, res) {

		var details = {
			'networkInterface' 	: req.body.networkInterface,
			'ethernet'	  	: req.body.ethernet,
			'wifi'		 	: req.body.wifi,
			'tcpip'	  	: req.body.tcpip,
			'proxy'			: req.body.proxy
		};

		app.brightSign.updateNetwork( details );

		res.render('applyingchanges', { 'networkUpdate' : JSON.stringify(details) });

	}.bind(this));


	this.app.post('/setup', this.checkAuth, function (req, res) {

		var networkDetails = {
			'networkInterface' 	: req.body.networkInterface,
			'ethernet'	  		: req.body.ethernet,
			'wifi'		 		: req.body.wifi,
			'tcpip'	  			: req.body.tcpip,
			'proxy'				: req.body.proxy
		};

		var authDetails = {
			'new' : {
				// 'username' : req.body.newUsername,
				'password' : req.body.newPassword
			},
			'old' : {
				// 'username' : req.body.currentUsername,
				'password' : req.body.currentPassword
			}
		};

		var multiOutputs = false;
		var resolution = false;

		if( app.brightSign.multipleOutputs )
		{
			multiOutputs = req.body.multiOutputs;
		}
		else 
		{
			resolution  = req.body.resolutionOverride;
		}
		
		// var audioOutput = req.body.audioOutput;

		setTimeout( function() {

			app.brightSign.applyConfig( networkDetails, authDetails, resolution, multiOutputs );
		
		}, 3000 );

		res.render('applyingchanges', { 'networkUpdate' : JSON.stringify(networkDetails) });

	}.bind(this));

	this.app.get('/wifilist', this.checkAuth, function (req, res) {

		var NetworkConfigClass = require("@brightsign/networkconfiguration");
		var networkConfigWifi = new NetworkConfigClass("wlan0");

		networkConfigWifi.scan().then(
			function(response) {
				res.json(response);
			}
		).catch(
			function(e) {
				res.json({'error' : true});
			}
		);

	}.bind(this));

	this.app.all('/status', function(req, res, next) {

		if( app.brightSign.currentNetworkConfig && app.brightSign.currentNetworkConfig.ipaddressold )
		{
			res.header("Access-Control-Allow-Origin", 'http://' + app.brightSign.currentNetworkConfig.ipaddressold + ':8000');
			res.header("Access-Control-Allow-Headers", "X-Requested-With");	
		}
		
		next();
	});

	this.app.get('/status', function (req, res) {

		res.send('Online');

	});

};

Settings.prototype.serverError = function(e)
{
	if( e.toString().indexOf('EADDRINUSE') != -1 )
	{
		app.brightSign.reboot();
	}
};




















